;******************************************************************************
;                      K S 1 2 A I N . A S M
;******************************************************************************
;This module checks for new A/D input data in the circular buffer "Sample_Que"
;and then processes the new data.  The analog data is processed into a serial
;digital data stream for decoding.

;==============================================================================
;      ========          I n i t _ A i n ( )     ==========
;==============================================================================
Init_Ain:                       ;initialize variables
        lrlk    AR6,Sample_Que  ;init circular A/D & D/A buffer pointers
        lrlk    AR5,Sample_Que
        lrlk    AR4,Sample_Que+1
;
        lalk    DELAY_CONST
        sacl    DelyK
        lalk    ADLIMIT
        sacl    AD_Limit
        lalk    INC1200
        sacl    Phz_Inc_1200
        zac
        sacl    Position_Sum
        sacl    Position_Ave
;
;Load FIR Filter Coefficients into B0 Page memory
        cnfd                    ;map B0 to data memory
        larp    AR0             ;ARP = AR0 = delay line pointer
        lrlk    AR0,BPFIRCOEF
        rptk    BPFIR_LENGTH-1  ;load BP FIR filter coefficients
        blkp    BPFCOEF,*+
        cnfp                    ;map B0 to program memory
;
        cnfd                    ;map B0 to data memory
        larp    AR0             ;ARP = AR0 = delay line pointer
        lrlk    AR0,LPFIRCOEF
        rptk    LPFIR_LENGTH-1  ;load LP FIR filter coefficients
        blkp    LPFCOEF,*+
        cnfp                    ;map B0 to program memory
        ret
;..............................................................................
;==============================================================================
;        ========     S e r v i c e _ A i n ( )     ==========
;==============================================================================
;void Service_Ain()
;{
;  if( AR5 != AR6 ){            // if receive Sample_Que is not empty
;     Temp1 = Sample_Que[ AR5++ ];
;     if( abs( Temp1 ) > AD_Limit ){  //indicate clipping
;        Flags.PEAK_DET = TRUE;
;        Timer2 = ONESHOT_LEN;
;        LED6on();
;     }
;     if( AR5 > Sample_Que + SAMPQUESIZE-1 )
;        AR5 = Sample_Que;
;     ProcessInput(Temp1);      // process new input sample
;  }
;  else{
;     if( Flags.PEAK_DET ){
;        if( !Timer2 ){
;           Flags.PEAK_DET = FALSE;
;           LED6off();
;        }
;     }
;  }
;}
;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
Service_Ain:                    ;service input samples from A/D
        larp    AR5             ;ARP = Sample_Que tail pointer
        sar     AR6,Temp0       
        lar     AR0,Temp0       ;Temp0 = head
        cmpr    CMEQ            ;see if head == tail   TC = (AR6==AR5)
        bbnz    SA_2            ;br if Sample_Que is empty
        larp    AR5             ;get sample
        lac     *+,14
        sach    Temp1           ;input range is 14 bits(+/- 8192)
;
        lac     Temp1
        abs
        sub     AD_Limit
        blz     SA_4            ;br if |signal| less than the threshold limit
        lac     Flags
        ork     PEAK_DET        ;latch peak event
        sacl    Flags
        lack    ONESHOT_LEN     ;start LED6 pulse stretcher
        sacl    Timer2
        call    LED6on
;
SA_4:   lrlk    AR0,Sample_Que+SAMPQUESIZE-1    ;check for wrap around
        cmpr    CMGT            ;TC = (AR5>AR0)
        bbz     SA_1            ;br if no wrap
        lrlk    AR5,Sample_Que  ;else wrap it around
SA_1:   call    ProcessInput    ;go process next A/D input sample in Temp1
        ret
SA_2:   bit     Flags,T_PEAK_DET
        bbz     SA_3            ;if peak det LED is on then
        lac     Timer2          ;check timeout
        bnz     SA_3
        lac     Flags
        andk    ~PEAK_DET
        sacl    Flags
        call    LED6off
SA_3:   ret
;
;==============================================================================
;      =======   P r o c e s s I n p u t ( Temp1 )     =======
;==============================================================================
;This routine is called every A/D sample and processes the input
;sample.  The input is bandpass filtered, sent through a frequency
;discriminator, then through a lowpass filter.  Binary Data is then
;extracted with a data slicer.  Data transition edges are used to
;derive a phase locked data clock by calling Phz_Lock().  A carrier
;detect function is implemented using the phase error of the data clock.
;  If no edges are detected within DCD_TIMEOUT samples, the carrier detect
;is turned off.
;
;void ProcessInput( Temp1 )
;{
;   BPFdelay[ 0 ] = Temp1;      // do a FIR calculation
;   acc = 0;
;   for( i=BPFIR_LENGTH-1; i >= 0; i-- ){
;      acc += BPFdelay[ i ] * BPFIRCOEF[ i ];
;      BPFdelay[ i+1 ] = BPFdelay[ i ];
;   }
;   Temp0 = acc;                // Temp0 = bandpass output data 
;
;   P = Temp0 * -( Dely0 + Dely1 * DELAY_CONST * 2.0 );
;   Dely1 = Dely0;
;   Dely0 = Temp0/2;
;   Temp0 = P * 2;             // Temp0 = discriminator output*2
;
;   LPFdelay[ 0 ] = Temp0;
;   acc = 0;
;   for( i=LPFIR_LENGTH-1; i >= 0; i-- ){
;      acc += LPFdelay[ i ] * LPFIRCOEF[ i ];
;      LPFdelay[ i+1 ] = LPFdelay[ i ];
;   }
;   Temp0 = acc;                // Temp0 = lowpass output data
;
;   if( Temp0 > SLICER_HYST && !Flags.INPUTBIT ){     // if + edge 
;       Flags.INPUTBIT = TRUE;
;       RD_Ion();
;       Phz_Lock();
;   }
;   else{
;      if(( Temp0 < -SLICER_HYST && Flags.INPUTBIT ){ // if - edge
;       Flags.INPUTBIT = FALSE;
;       RD_Ioff();
;       Phz_Lock();
;      }
;      else{                    // if no transitions for a while then
;         if( DCD_OneShot > 0 ) // turn off carrier detect
;            DCD_OneShot--;
;         else{
;            DCD_Counter = 0;
;            Flags.CARRIERDET = FALSE;
;            LED7off();
;            LED4on();
;         }
;      }
;   }
;   if( Recv_Phz_Acc < 0 && (Recv_Phz_Acc += INC1200) >= 0 ){
;      RD_Clkon();              // if 1200 bps time
;      Process_Inbits();        // go assemble the bits
;      RD_Clkoff();
;   }
;}
;
ProcessInput:
        larp    AR0             ;ARP = AR0 = delay line pointer
        lrlk    AR0,BPFdelay    ;AR0 points to start of delay line
        lac     Temp1
        sacl    *               ;store new sample at beginning of buf
        adrk    BPFIR_LENGTH-1  ;point to end of delay line
        mpyk    0		;initialize sum and product registers
        zac
        rptk    BPFIR_LENGTH-1  ;multiply and accumulate
        macd    (BPFIRCOEF+B0PADDR-B0DADDR),*-
        apac                    ; include last product
        sach    Temp0           ;filter gain = 2.9
;
;$$$$$$$$$$$$$$$$
;        lac     Temp0
;        andk    0FFFCh
;        sacl    TestOut         ;TEST OUT
;$$$$$$$$$$$$$$$$
        lt      Dely1           ;get oldest sample
        mpy     DelyK           ;P = Dely1 * DELAY_CONST
        pac
        sach    Temp1           ;Temp1 = Dely1 * DELAY_CONST
        lac     Temp1,1         ;acc = Temp1*2
        add     Dely0           ;add to second oldest sample
        neg                     ;acc = -( Dely0 + Dely1 * DELAY_CONST )
        sacl    Temp1           ;Temp1 = acc
        lt      Temp1
        mpy     Temp0           ;P = Temp0 * -( Dely1 + Dely0 * DELAY_CONST )
;
        lac     Dely0
        sacl    Dely1           ;Dely1 = Dely0
        lac     Temp0,15
        sach    Dely0           ;Dely0 = Temp0/2
        pac
        sach    Temp0,1         ;Mult discrim output by 2
;
                                ;exe time = 5.5uSec
;$$$$$$$$$$$$$$$$
;        lac     Temp0
;        andk    0FFFCh
;        sacl    TestOut         ;TEST OUT
;$$$$$$$$$$$$$$$$
        lrlk    AR0,LPFdelay    ;AR0 points to start of delay line
        lac     Temp0
        sacl    *               ;store new sample at beginning of buf
        adrk    LPFIR_LENGTH-1  ;point to end of delay line
        mpyk    0		;initialize sum and product registers
        zac
        rptk    LPFIR_LENGTH-1  ;multiply and accumulate
        macd    (LPFIRCOEF+B0PADDR-B0DADDR),*-
        apac                    ; include last product
        sach    Temp0
;
;$$$$$$$$$$$$$$$$
;        lac     Temp0
;        andk    0FFFCh
;        sacl    TestOut         ;TEST OUT
;$$$$$$$$$$$$$$$$
        lac     Temp0
        blz     PI_NEG
PI_POS:
        subk    SLICER_HYST     ;Ignor data near -0
        bgz     PI_1
        b       PI_3
PI_NEG:
        addk    SLICER_HYST     ;Ignor data near +0
        blz     PI_2
        b       PI_3
PI_1:                           ;here if data is high
        bit     Flags,T_INPUTBIT
        bbnz    PI_3
        lac     Flags
        ork     INPUTBIT        ;else Flags.INPUTBIT = TRUE;
        sacl    Flags
        call    RD_Ion
        call    Phz_Lock        ;do phaze lock on + edge
        b       PI_5
PI_2:                           ;here if data is low
        bit     Flags,T_INPUTBIT
        bbz     PI_3
        lac     Flags
        andk    ~INPUTBIT       ;else Flags.INPUTBIT = FALSE;
        sacl    Flags
        call    RD_Ioff
        call    Phz_Lock        ;do phaze lock on - edge
        b       PI_5
PI_3:                           ;here if no transitions
        lac     DCD_OneShot     ;decrement Oneshot counter to
        bz      PI_4            ;turn off carrier detect if there is no
        subk    1               ;transitions for a while
        sacl    DCD_OneShot
        b       PI_5
PI_4:   
        zac                     ;turn off carrier detect if no transitions
        sacl    DCD_Counter
        lac     Flags
        andk    ~CARRIERDET     ;else Flags.CARRIERDET = FALSE;
        sacl    Flags
        call    LED4on
        call    LED7off
;
PI_5:                           ;See if 1200 Hz bit time
        lac     Recv_Phz_Acc    ;add phase increment for 1200bps
        bgez    PI_6            ;br if old is +
        add     Phz_Inc_1200    ;else old is - , add phase inc
        sacl    Recv_Phz_Acc
        bit     Recv_Phz_Acc,15
        bbnz    PI_7            ;br if new is still -
        call    RD_Clkon        ;else sign changed - to +
        call    Process_Inbits  ;process bit sample
        call    RD_Clkoff
        b       PI_7
PI_6    add     Phz_Inc_1200    ;old sign was +, add phase inc
        sacl    Recv_Phz_Acc
PI_7:   ret
;..............................................................................
;==============================================================================
;      =======            P h z _ L o c k ( )       =======
;==============================================================================
;   This routine is called every time a transition occurs on the data input.
;The Recv_Phz_Acc is incremented or decremented to phase lock the clock to the
;data using a lookup table. Only the upper 4 MS bits are used for position.
;  The phase lock lines up the Recv_Phz_Acc that the bit edges are at count
;0x7000 and the center of a bit is at count 0x0000.
;   The Carrier detect counter is incremented if the edge occurs within
;a specified window and decremented to zero if not.  If the counter reaches
;the CAR_THRESHOLD, then the carrier detect flag is set to start collecting
;the incoming data. If the counter falls to zero then the carrier detect
;flag is cleared.  Two tables are used to calculate the increment and
;decrement values depending on the state of the carrier detect and the
;clock position.
;
;void Phz_Lock(void)
;{
;   Temp1 = Recv_Phz_Acc >> 12;         //use upper 4 bits as clock position
;   if( Flags.CARRIERDET == TRUE)
;      Temp0 = DCD_TABLE_SLOW[ Temp1 ]; //use slow table after lockup
;   else
;      Temp0 = DCD_TABLE_FAST[ Temp1 ]; //use fast table before lockup
;   DCD_Counter += Temp0;
;   if( DCD_Counter <= 0 ){             // clamp DCD_Counter to zero
;      Flags.CARRIERDET = FALSE;
;      DCD_Counter = 0;
;      LED7off();
;      LED4on();
;   }
;   if( DCD_Counter >= CAR_THRESHOLD ){ // clamp DCD_Counter to max value
;      Flags.CARRIERDET = TRUE;
;      DCD_Counter = CAR_THRESHOLD;
;      LED7on();
;      LED4off();
;   }
;   Position_Sum = Temp1 - Position_Ave; //smooth out the position a little
;   Position_Ave = Position_Sum/4;       // with cheap and dirty average
;   Recv_Phz_Acc += CLK_ERROR_TABLE[ Posiion_Ave ]; // perform phase lock using
;                                                   // table lookup
;   DCD_Oneshot = DCD_TIMEOUT;          // reset DCD_oneshot timer
;}
;
Phz_Lock:
        rsxm
        lac     Recv_Phz_Acc,4          ;get upper four bits
        ssxm
        sach    Temp1                   ;Temp1=index  range=0toF
;        
        zals    Temp1
        bit     Flags,T_CARRIERDET
        bbz     PL_3
        adlk    DCD_TABLE_SLOW
        b       PL_4
PL_3:
        adlk    DCD_TABLE_FAST
PL_4:   tblr    Temp0           ;get DCD update value from table
        lac     DCD_Counter
        add     Temp0
        sacl    DCD_Counter
;
        subk    CAR_THRESHOLD
        blz     PL_1            ;br if( DCD_Counter < CAR_THRESHOLD )
        lack    CAR_THRESHOLD
        sacl    DCD_Counter
        lac     Flags
        ork     CARRIERDET      ;Flags.CARRIERDET = TRUE;
        sacl    Flags
        call    LED7on
        call    LED4off
        b       PL_2
PL_1:                           ;here if( DCD_Counter < CAR_THRESHOLD )
        lac     DCD_Counter
        bgz     PL_2            ;br if( DCD_Counter > 0 )
        zac
        sacl    DCD_Counter
        lac     Flags
        andk    ~CARRIERDET     ;else Flags.CARRIERDET = FALSE;
        sacl    Flags
        call    LED7off
        call    LED4on
PL_2:
        lac     Position_Sum    ;average over 4 samples
        sub     Position_Ave
        add     Temp1
        sacl    Position_Sum
        sfr
        sfr
        sacl    Position_Ave
;
        adlk    CLK_ERROR_TABLE
        tblr    Temp1           ;get updated Recv_Phz_Acc from table
        zals    Recv_Phz_Acc
        add     Temp1
        sacl    Recv_Phz_Acc
;
        lack    DCD_TIMEOUT     ;reset DCD One shot timer
        sacl    DCD_OneShot
        ret
;..............................................................................
;==============================================================================
;       =======     P r o c e s s _ I n b i t s ( )       =======
;==============================================================================
;   This routine is called every data clock sample time to assemble the serial
;NRZI data stream into flag and data bytes.
;
;void Process_Inbits( Flags )
;   if( Flags.CARRIERDET ){            // if carrier present
;      if( Flags.RXLASTBIT == Flags.INPUTBIT ){  
;         if( RX_OnesCounter < 5 ){    // Input Bit is same as last so is ONE
;            RX_OnesCounter++;         // shift in valid ONE data bit
;            RX_ByteRegister >>= 1;
;            RX_ByteRegister |= 80h;
;            if( ++RX_BitCounter >= BITSINBYTE ){  // got a whole byte?
;               RX_BitCounter = 0;
;               RX_NewByte = RX_ByteRegister;
;               Flags.NEWBYTERDY = TRUE;
;            }
;         }
;         else{
;            if( RX_OnesCounter < MAX_ONES ){ // clamp ones_counter to MAX
;               RX_OnesCounter++;
;            }
;         }
;      }
;      else{            // Input bit changed polarity so is a ZERO data bit
;         Flags.RXLASTBIT = Flags.INPUTBIT;
;         if( RX_OnesCounter < 5 ){    // is not a flag or stuffed bit yet
;            RX_ByteRegister >>= 1;    // shift in a zero
;            RX_OnesCounter = 0;
;            if( ++RX_BitCounter >= BITSINBYTE ){  // got a whole byte?
;               RX_BitCounter = 0;
;               RX_NewByte = RX_ByteRegister;
;               Flags.NEWBYTERDY = TRUE;
;            }
;         }
;         else{                   
;            if( RX_OnesCounter == 6 ){   // is a flag so mark it as such
;               RX_BitCounter = 0;
;               RX_NewByte = HDLC_FLAG;
;               Flags.NEWFRAMERDY = TRUE;
;               Flags.NEWBYTERDY = TRUE;
;            }                // else is a stuffed bit so ignore this zero
;            RX_OnesCounter = 0;
;         }
;      }
;   }
;   else{                               // here if carrier goes away
;      Flags.NEWBYTERDY = TRUE;         // reset receiver variables
;      Flags.NEWFLAGRDY = TRUE;
;      RX_BitCounter = 0;
;      RX_OnesCounter = 0;
;      RX_ByteRegister = 0;
;   }
;}
Process_Inbits:
        bit     Flags,T_CARRIERDET      ;if carrier is detected
        bbz     PIb_9                   ;br if not
        bit     Flags,T_INPUTBIT        ;TC = input bit
        bbz     PIb_2                   ;b if input bit == 0
                                        ;here if input bit == 1
        bit     Flags,T_RXLASTBIT       ;TC = previous input bit
        lac     Flags
        ork     RXLASTBIT               ;Flags.RXLASTBIT = Flags.INPUTBIT
        sacl    Flags
        bbz     PIb_4                   ;b if previous bit == 0
        b       PIb_3
PIb_2:                                  ;here if input bit == 0
        bit     Flags,T_RXLASTBIT       ;TC = previous input bit
        lac     Flags
        andk    ~RXLASTBIT              ;Flags.RXLASTBIT = Flags.INPUTBIT
        sacl    Flags
        bbnz    PIb_4                   ;b if previous bit == 1
PIb_3:                                  ;here if( input bit == previous bit)
                                        ;so data is a ONE
        lac     RX_OnesCounter         
        subk    5
        bgez    PIb_5                   ;br if( RX_OnesCounter >= 5 )
        addk    6                       ;RX_OnesCounter++
        sacl    RX_OnesCounter
        lac     RX_ByteRegister         ;RX_ByteRegister >>= 1
        sfr                             ;shift in a one
        ork     080h                    ;RX_ByteRegister |= 80h;
        sacl    RX_ByteRegister
        b       PIb_6
PIb_5:
        lac     RX_OnesCounter         
        subk    MAX_ONES                ;if( RX_OnesCounter < MAX_ONES )
        bgez    PIb_1                   ;br if( RX_OnesCounter >= MAX_ONES )
        addk    MAX_ONES+1              ;RX_OnesCounter++;
        sacl    RX_OnesCounter
        b       PIb_1
PIb_4:                                  ;here if( input bit != previous bit)
                                        ;so input data is a ZERO
        lac     RX_OnesCounter         
        subk    5
        bgez    PIb_7                   ;br if( RX_OnesCounter >= 5 )
        zac                             ;RX_OnesCounter = 0
        sacl    RX_OnesCounter
        lac     RX_ByteRegister         ;RX_ByteRegister >>= 1
        sfr                             ;shift in a zero
        sacl    RX_ByteRegister
        b       PIb_6
PIb_7:
        subk    1                       ;acc = RX_OnesCounter-5-1
        bnz     PIb_8                   ;br if(RX_OnesCounter != 6)
        zac
        sacl    RX_BitCounter           ;RX_BitCounter = 0;
        lack    HDLC_FLAG
        sacl    RX_NewByte              ;RX_NewByte = HDLC_FLAG;
        lac     Flags
        ork     NEWFLAGRDY|NEWBYTERDY   ;Flags.NEWFLAGRDY = TRUE;
        sacl    Flags                   ;Flags.NEWBYTERDY = TRUE;
PIb_8:  zac                             ;RX_OnesCounter = 0
        sacl    RX_OnesCounter
        b       PIb_1
;
PIb_6:
        lac     RX_BitCounter           ;if( ++RX_BitCounter >= BITSINBYTE ){
        addk    1                       ;++RX_BitCounter
        sacl    RX_BitCounter
        subk    BITSINBYTE
        blz     PIb_1                   ;br if( ++RX_BitCounter < BITSINBYTE )
        zac
        sacl    RX_BitCounter           ;RX_BitCounter = 0
        lac     RX_ByteRegister
        sacl    RX_NewByte              ;RX_NewByte = RX_ByteRegister;
        lac     Flags
        ork     NEWBYTERDY              ;Flags.NEWBYTERDY = TRUE;
        sacl    Flags
PIb_1:  ret
PIb_9:                                  ;here if no carrier
        zac
        sacl    RX_BitCounter           ;RX_BitCounter = 0;
        sacl    RX_OnesCounter          ;RX_OnesCounter = 0;
        sacl    RX_ByteRegister         ;RX_ByteRegister = 0;
        lac     Flags                   ;Flags.NEWBYTERDY = TRUE; 
        andk    ~(NEWFLAGRDY|NEWBYTERDY)
        sacl    Flags                   ;Flags.NEWFLAGRDY = TRUE;
        ret
;....................E N D   o f   K S 1 2 A I N . A S M ......................
