;******************************************************************************
;                      K S 1 2 A O U T . A S M
;******************************************************************************
;This module takes a byte of digital data and converts it into an HDLC serial
;data stream.  A new D/A output sample is calculated and placed in the circular
;buffer "Sample_Que".

;==============================================================================
;       =========      S e r v i c e _ A o u t ( )    =========
;==============================================================================
;Put samples into the D/A converter circular buffer for transmitting.
;void Service_Aout()
;{
;  if( AR4 != AR5 ){            // if transmit Sample_Que is not empty
;     Sample_Que[ AR4++ ] = OutBuf;
;     if( AR4 > Sample_Que + SAMPQUESIZE-1 )
;        AR4 = Sample_Que;
;     if( Xmit_Phz_Acc > 0 && (Xmit_Phz_Acc += INC1200) < 0 ){
;        BitTransmit();         // call when sign of Xmit_Phz_Acc changes
;     }
;     OutBuf = SIN_TABLE[ PhaseAcc >> 8 ];
;     PhaseAcc = ( PhaseAcc + PhaseInc ) & 07FFFh;
;  }
;}
;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
Service_Aout:                   ;service sending of output samples to D/A
        larp    AR4             ;ARP = Sample_Que xmit head pointer
        sar     AR5,Temp0       ;
        lar     AR0,Temp0       ;Temp0 = receive tail
        cmpr                    ;see if head == tail
        bbnz    So4             ;br to exit if Sample_Que is full
        lac     OutBuf
;
        sacl    *+,2            ;put next output data in Sample_Que and
                                ;inc head pointer
        lrlk    AR0,Sample_Que+SAMPQUESIZE-1    ;check for wrap around
        cmpr    CMGT            ;TC= (AR4>AR0)
        bbz     So1             ;br if no wrap
        lrlk    AR4,Sample_Que  ;else wrap it around
So1:                            ;See if 1200 Hz bit time
        lac     Xmit_Phz_Acc    ;add phase increment for 1200bps
        blz     So2             ;br if old is -
        add     Phz_Inc_1200    ;else old is + , add phase inc
        sacl    Xmit_Phz_Acc
        bit     Xmit_Phz_Acc,15
        bbz     So3             ;br if new is still +
        sacl    Xmit_Phz_Acc    ;else sign changed + to -
        call    BitTransmit     ;calculate the next data bit output value
        b       So3
So2:    add     Phz_Inc_1200    ;sign was -, add phase inc
        sacl    Xmit_Phz_Acc
So3:
;. . . . . .   Calculate next D/A value for tone generator  . . . . .
        lac     PhaseAcc,8      ;get current phase accumulator value
        sach    OutBuf          ;OutBuf=top 7 msbits
        lac     OutBuf
        adlk    SIN_TABLE       ;add phase to SIN table start address
        tblr    OutBuf          ;get next D/A value in from table
        lac     PhaseAcc        ;get current phase accumulator value
        add     PhaseInc        ;add phase inc value to accumulator
        andk    07FFFh          ;truncate so top byte range is 0-127
        sacl    PhaseAcc        ;store new phase accumulator value
So4:    ret
;==============================================================================
;    ===========       B i t T r a n s m i t ( )        =============
;==============================================================================
;   Called at 1200 Hz rate to determine MARK or SPACE frequency depending on
;data stream.  Inputs to this routine are "NewByte" which holds the next byte
;to send and Flags.SENDFLAG which is set if a flag byte is to be sent.
;
;void BitTransmit()
;{
;   if( Flags.TRANSMITON ){
;      if( (TX_ByteRegister & 1) == 1 ){
;         TX_OnesCounter++;
;         if( TX_OnesCounter != 5 ){
;            if( TX_OnesCounter > 5 ){  /* see if need to ZERO insert    */
;               TX_OnesCounter = 0;
;               Flags.TXLASTBIT = ~Flags.TXLASTBIT;
;               Flags.TXPRESENTBIT = Flags.TXLASTBIT;
;            }
;            nextbit();
;         }
;      }
;      else{                            /* if data bit is a ZERO             */
;         Flags.TXLASTBIT = ~Flags.TXLASTBIT;
;         Flags.TXPRESENTBIT = Flags.TXLASTBIT;
;         if( flags.FLAGING ){          /* allow 6 ONE bits to be sent       */
;            TX_OnesCounter = -5;
;            flags.FLAGING = FALSE;
;         }
;         else
;            TX_OnesCounter = 0;
;         }
;         nextbit();
;      }
;      if( Flags.PRESENTBIT )
;         PhaseInc = INCH;              /* set 2200 Hz tone */
;      else
;         PhaseInc = INCL;              /* set 1200 Hz tone */
;   }
;   else
;      PhaseInc = 0;                    /* turn off tone */
;}
;void nextbit()
;{
;   TX_ByteRegister >>= 1;
;   if( ++TX_BitCounter >= BITSINBYTE ){
;      TX_BitCounter = 0;
;      if( Flags.SENDFLAG )
;         Flags.FLAGING = TRUE;
;      Flags.SENDFLAG = FALSE;
;      TX_ByteRegister = TX_NewByte;
;      Flags.NEXTBYTE = TRUE;   /*signal ok to load next byte*/
;   }
;}
;.............................................................................
BitTransmit:
        bit     Flags,T_TRANSMITON      ;if transmiter is on
        bbz     BT1                     ;br if not
        bit     TX_ByteRegister,0       ;read data bit
        bbz     BT2                     ;br if data bit is zero
        lac     TX_OnesCounter
        addk    1
        sacl    TX_OnesCounter
        subk    5
        bz      BT10                    ;br if TX_OnesCounter==5               
        blez    BT3                     ;br if TX_OnesCounter<5        
        zac
        sacl    TX_OnesCounter          ;TX_OnesCounter=0
        lac     Flags
        xork    TXLASTBIT               ;Flags.TXLASTBIT = ~Flags.TXLASTBIT
        sacl    Flags
        bit     Flags,T_TXLASTBIT       ;Flags.TXPRESENTBIT = Flags.TXLASTBIT
        bbz     BT4
        ork     TXPRESENTBIT
        b       BT5
BT4:    andk    ~TXPRESENTBIT
BT5:    sacl    Flags
        b       BT3
;
BT2:    lac     Flags                   ;here if data bit is a zero
        xork    TXLASTBIT               ;Flags.TXLASTBIT = ~Flags.TXLASTBIT
        sacl    Flags
        bit     Flags,T_TXLASTBIT       ;Flags.TXPRESENTBIT = Flags.TXLASTBIT
        bbz     BT6
        ork     TXPRESENTBIT
        b       BT7
BT6:    andk    ~TXPRESENTBIT
BT7:    sacl    Flags
;
        zac
        bit     Flags,T_FLAGING         ;see if a flag byte
        bbz     BT8                     ;br if not
        lack    5
        neg
BT8:    sacl    TX_OnesCounter
        lac     Flags
        andk    ~FLAGING
        sacl    Flags
;
BT3:    lac     TX_ByteRegister         ;nextbit()
        ror
        sacl    TX_ByteRegister         ;TX_ByteRegister>>1
        lac     TX_BitCounter
        addk    1
        sacl    TX_BitCounter
        subk    BITSINBYTE              ;see if last bit in byte        
        blz     BT10                    ;br if not last bit
        zac                             ;else TX_BitCounter=0
        sacl    TX_BitCounter
        lac     Flags
        bit     Flags,T_SENDFLAG        ;see if need to send flag
        bbz     BT9                     ;br if FALSE
        ork     FLAGING                 ;Flags.FLAGING = TRUE;
BT9:    andk    ~SENDFLAG               ;Flags.SENDFLAG = FALSE
        ork     NEXTBYTE                ;Flags.NEXTBYTE = TRUE;
        sacl    Flags
        lac     TX_NewByte
        sacl    TX_ByteRegister         ;TX_ByteRegister = TX_NewByte
BT10:   lalk    INCL
        bit     Flags,T_TXPRESENTBIT
        bbz     BT12
        lalk    INCH
        b       BT11
BT12:
        lalk    INCL
        b       BT11
BT1:    zac                             ;here if xmitter is off
BT11:   sacl    PhaseInc                ;keep tones off
        ret

;..................E N D   o f   K S 1 2 A O U T . A S M ......................
