;******************************************************************************
;                      K S 1 2 H I N . A S M
;******************************************************************************
;   This module gets bytes of HDLC data from the 1200 BPS modem and places the
;data in the circular buffer "KSout_Que".  When a flag byte is received
;the checksum is checked and if ok, then the packet is signaled complete.
;
;==============================================================================
;      ========          I n i t _ H i n ( )     ==========
;==============================================================================
Init_Hin:                       ;initialize variables
        zac
        sacl    Recv_Phz_Acc
        sacl    DCD_Counter
        sacl    RX_BitCounter           ;RX_BitCounter = 0;
        sacl    RX_OnesCounter          ;RX_OnesCounter = 0;
        sacl    RX_ByteRegister         ;RX_ByteRegister = 0;
        lalk    GOOD_CRC
        sacl    GoodCRC
        lalk    Wait_First_Flag         ;set next state
        sacl    Hin_Vector
        ret
;..............................................................................
;==============================================================================
;       ========          S e r v i c e _ H i n ( )     ==========
;==============================================================================
;void Service_Hin()
:{
;   Hin_Vector();     // call next state
;}
;
Service_Hin:
        lac     Hin_Vector
        cala
        ret
;..............................................................................

;==============================================================================
;     ========     W a i t _ F i r s t _ F l a g ( )       ==========
;==============================================================================
;    This routine checks the Flags.NEWFLAGRDY for the first flag to be
;received then dispatches the next state Wait_More_Flags to wait for any more
;flag bytes before beginning a new packet reception.
;
;void Wait_First_Flag( void )
;{
;   if( Flags.NEWFLAGRDY ){            // if new flag byte is received
;      Flags.NEWFLAGRDY = FALSE;
;      Flags.NEWBYTERDY = FALSE;
;      Hin_Vector = Wait_More_Flags;
;   }
;}
;.............................................................................
Wait_First_Flag:
        bit     Flags,T_NEWFLAGRDY
        bbz     Wait_F_1
        lac     Flags
        andk    ~(NEWFLAGRDY|NEWBYTERDY)
        sacl    Flags
        lalk    Wait_More_Flags        ;set next state
        sacl    Hin_Vector
Wait_F_1:
        ret
;.............................................................................
;==============================================================================
;      ========     W a i t _ M o r e _ F l a g s ( )       ==========
;==============================================================================
;    This routine checks the Flags.NEWFLAGSRDY for any more flags to be
;received.  When the first non-flag byte is received it dispatches the next
;state Put_Packet_Data to begin a new data packet reception.
;
;void Wait_More_Flags( void )
;{
;   if( Flags.NEWFLAGRDY ){             // if another flag byte 
;      Flags.NEWFLAGRDY = FALSE;        // just another flag so keep waiting
;      Flags.NEWBYTERDY = FALSE;
;   else{
;      if( Flags.NEWBYTERDY ){          // if first data byte ready
;         RX_crc = 0xFFFF;              // initialize checksum
;         KSout_Start = KSout_Head;     // save data starting location
;         Hin_Vector = Put_Packet_Data;
;      }
;   }
;   if( !Flags.CARRIERDET )             // if carrier lost
;      Hin_Vector = Wait_First_Flag;    // wait for new flag byte
;}
;.............................................................................
Wait_More_Flags:
        bit     Flags,T_NEWFLAGRDY
        bbnz    Wait_MF_1               ;br if was another flag byte
;
        bit     Flags,T_NEWBYTERDY
        bbz     Wait_MF_2               ;br if not a new byte
        zac                             ;esle was the first data byte
        cmpl
        sacl    RX_crc
        lar     AR0,KSout_Head
        sar     AR0,KSout_Start
        lalk    Put_Packet_Data         ;set next state
        sacl    Hin_Vector
        ret
Wait_MF_1:
        lac     Flags
        andk    ~(NEWFLAGRDY|NEWBYTERDY)
        sacl    Flags
Wait_MF_2:
        bit     Flags,T_CARRIERDET      ;if( !Flags.CARRIERDET )
        bbnz    Wait_MF_3               ;br if still have carrier
        lalk    Wait_First_Flag         ;preload 
        sacl    Hin_Vector
Wait_MF_3:
        ret
;.............................................................................
;==============================================================================
;       ========     P u t _ P a c k e t _ D a t a ( )       ==========
;==============================================================================
;   This waits for a new data byte "RX_NewByte" to be received and places it
;into the KSout_Que.  The checksum is calculated for each byte.  If a flag is
;received the RX_crc is checked for validity.  If everything is OK then the
;head pointer KSout_Head is moved back two bytes to ignor the checksum bytes
;in the packet since they aren't needed any more.  The first byte in the packet
;is then tagged as ready to be sent out the UART link in KISS format by the
;KS12kout module.
;  If carrier is lost before a end flag is received, the pointers are reset
;back to the start.  If a bad CRC is found the same thing is done.
;
;void Put_Packet_Data(void)
;{
;   if( Flags.CARRIERDET ){             // if carrier still detected
;      if( Flags.NEWBYTERDY ){          // if a new byte is received
;         Flags.NEWBYTERDY = FALSE;
;         if( Flags.NEWFLAGRDY ){       // if an ending flag byte is detected
;            Flags.NEWFLAGRDY = FALSE;
;            if( RX_crc == GoodCRC ){
;               KSout_Que[ KSout_Head ] = 0;    // backup over crc bytes
;               if( KSout_Head == KSout_Que )    // handle wrap around
;                  KSout_Head = KSout_Que + OUTQUESIZE-1;
;               else
;                  KSout_Head--;
;               KSout_Que[ KSout_Head ] = 0;
;               KSout_Que[ KSout_Start ] = PACKETDONE;  // signal packet ready
;            }
;            else{                         // if crc was bad
;               KSout_Head = KSout_Start;  // put head pointer back to start
;               KSout_Que[ KSout_Head ] = 0;
;            }
;            Hin_Vector = Wait_More_Flags; //wait for another flag byte
;         }
;         else{                 // is another data byte so put in the QUE
;            Calc_RXcrc();
;            if( ++KSout_Head > KSin_Que+OUTQUESIZE-1 ) // handle wrap around
;               KSout_Head = KSout_Que;
;            KSout_Que[ KSout_Head ] = RX_NewByte | DATAVALID;
;         }
;      }
;   }
;   else{                               // here if lost carrier
;      KSout_Head = KSout_Start;        // put head pointer back to beginning
;      KSout_Que[ KSout_Head ] = 0;
;      Hin_Vector = Wait_First_Flag;   // wait for new flag byte
;   }
;}
;.  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
;                            Initial state of
;                              KSout_Que[] 
;                          --------------------
;                         |         :          |
;                          --------------------
;    KSout_Start--->      |    0    :    0     |
;    KSout_Head            --------------------
;                         |    x    :    x     |
;                          --------------------
;                         |    x    :    x     |
;                          --------------------
;                         |    x    :    x     |
;                          --------------------
;                         |         :          |
;                          --------------------
;
;.  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
;                           While entering data
;                              KSout_Que[] 
;                          --------------------
;                         |         :          |
;                          --------------------
;    KSout_Start--->      |    0    :    0     |  start of packet
;                          --------------------
;                         |    1    :   D0     |  1st data byte
;                          --------------------
;                         |    1    :   D1     |  2nd data byte
;                          --------------------
;     KSout_Head---->     |    1    :   D2     |  3rd data byte
;                          --------------------
;                         |         :          |  location for next data byte
;                          --------------------
;
;.  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
;                        when packet is complete
;                              KSout_Que[] 
;                          --------------------
;                         |         :          |
;                          --------------------
;    KSout_Start--->      |    2    :    0     |  start of packet
;                          --------------------
;                         |    1    :   D0     |  1st data byte
;                          --------------------
;                         |    1    :   D1     |  2nd data byte
;                          --------------------
;     KSout_Head---->     |    0    :    0     |  crclsb used to be here
;                          --------------------
;                         |    1    : crcmsb   |  crcmsb 
;                          --------------------
;
;
Put_Packet_Data:
        larp    AR1                     ;ARP = AR1
        bit     Flags,T_CARRIERDET      ;if( Flags.CARRIERDET )
        lalk    Wait_First_Flag         ;preload next state if needed
        bbz     PPD_2                   ;br if lost carrier
;
        bit     Flags,T_NEWBYTERDY      ;if( Flags.NEWBYTERDY )
        bbz     PPD_1
        lac     Flags
        andk    ~NEWBYTERDY
        sacl    Flags
;
        bit     Flags,T_NEWFLAGRDY      ;if( Flags.NEWFLAGRDY )
        bbz     PPD_3
        lac     Flags                   ;here if an ending flag
        andk    ~NEWFLAGRDY             ;Flags.NEWFLAGRDY = FALSE
        sacl    Flags
;
        zals    RX_crc                  ;if( RX_crc == GoodCRC )
        subs    GoodCRC
        bnz     PPD_7                   ;br if not
                                        ;here if good packet was received
        zac
        lar     AR1,KSout_Head
        sacl    *                       ;KSout_Que[ KSout_Head ] = 0
        lrlk    AR0,KSout_Que           ;check for wrap around
        cmpr    CMEQ                    ;TC = (AR1==AR0)
        bbz     PPD_5                   ;br if( KSout_Head != KSout_Que )
        lrlk    AR1,KSout_Que+OUTQUESIZE-1 ;else wrap it back around
        b       PPD_6
PPD_5:  mar     *-                      ;KSout_Head--
PPD_6:  zac
        sar     AR1,KSout_Head
        sacl    *                       ;KSout_Que[ KSout_Head ] = 0
        lar     AR1,KSout_Start
        lalk    PACKETDONE
        sacl    *                       ;KSout_Que[ KSout_Start ] = PACKETDONE
        lalk    Wait_More_Flags         ;wait for another flag byte
        sacl    Hin_Vector
        ret
PPD_3:                          ;here if is another data byte so put in the QUE
        call    Calc_RXcrc
        lar     AR1,KSout_Head
        mar     *+              ;++KSout_Head
        lrlk    AR0,KSout_Que+OUTQUESIZE-1      ;check for wrap around
        cmpr    CMGT            ;TC = (AR1>AR0)
        bbz     PPD_4           ;br if no wrap
        lrlk    AR1,KSout_Que   ;else wrap it around
PPD_4:  sar     AR1,KSout_Head
        lac     RX_NewByte
        ork     DATAVALID
        sacl    *       ;KSout_Que[ KSout_Head ] = RX_NewByte | DATAVALID
        ret
;
PPD_7:  lalk    Wait_More_Flags         ;here if crc was bad
PPD_2:  sacl    Hin_Vector              ;here if lost carrier
        lar     AR1,KSout_Start
        sar     AR1,KSout_Head
        zac
        sacl    *                       ;KSout_Que[ KSout_Head ] = 0
PPD_1:
        ret
;......................................................................
;========================================================================
;                 C a l c _ R X c r c ( RX_NewByte )
;========================================================================
; This routine calculates the next receive CRC value ( RX_crc )from the
;data stored in RX_NewByte.
;
;void Calc_RXcrc( RX_NewByte )
;{
;   crc_Temp = RX_NewByte ^ RX_crc & 0xFF;
;   RX_crc >>= 8;
;   RX_crc &= 0x00FF;
;   RX_crc ^= CRC_TABLE[ crc_Temp ];
;}
;.......................................................................
Calc_RXcrc:
        lac     RX_NewByte      ;   crc_temp = RX_NewByte ^ RX_crc & 0xFF;
        xor     RX_crc
        andk    0FFh
        sacl    crc_temp
;
        zac                     ;   RX_crc >>= 8;
        rsxm                    ;   RX_crc &= 0x00FF;
        lac     RX_crc,8
        sach    RX_crc
        ssxm
;
        lac     crc_temp        ;   RX_crc ^= CRC_TABLE[ crc_temp ];
        adlk    CRC_TABLE
        tblr    crc_temp
        zals    crc_temp
        xor     RX_crc
        sacl    RX_crc
        ret
;......................................................................
;....................E N D   o f   K S 1 2 H I N . A S M ......................
