;******************************************************************************
;                      K S 1 2 H O U T . A S M
;******************************************************************************
;   This module waits for a Transmit packet to be placed in the circular
;buffer "KSin_Que" then transmits the bytes in the packet out the 1200 BPS
;modem channel in HDLC format.
;
;==============================================================================
;     ========          I n i t _ H o u t ( )     ==========
;==============================================================================
Init_Hout:                      ;initialize variables
        zac
        sacl    PhaseInc
        sacl    PhaseAcc
        sacl    TX_BitCounter
        sacl    TX_OnesCounter
        sacl    TX_ByteRegister
        lalk    Wait_In_Packet  ;set next state
        sacl    Hout_Vector
        ret
;..............................................................................
;==============================================================================
;     ========          S e r v i c e _ H o u t ( )     ==========
;==============================================================================
;void Service_Hout()
:{
;   Hout_Vector();     // call present state
;}
;
Service_Hout:
        lac     Hout_Vector
        cala
        ret
;..............................................................................
;==============================================================================
;     ========     W a i t _ I n _ P a c k e t ( )       ==========
;==============================================================================
; This routine checks the KSin_Que for a new packet then dispatches
; the next state for beginning a transmission depending on Duplex being
; Half(0) or Full( !0 ).
;
;void Wait_In_Packet( void )
;{
;   if( KSin_Que[ KSin_Tail ] & PACKETDONE ){ // if packet complete 
;      if( Duplex ){
;         Transmit_On();                // Turn on xmitter
;         Timer1 = TXDelay;             // start TXDelay timer
;         Send_Flags();                 // Start sending flags
;         Hout_Vector = Wait_TX_Delay;
;      }
;      else{
;         Hout_Vector = Wait_No_Carrier;
;      }
;   }
;}
;.............................................................................
Wait_In_Packet:
        call    LED3off
        larp    AR0             ;ARP = AR0
        lar     AR0,KSin_Tail
        lac     *               ;acc = KSin_Que[ KSin_Tail ]
        andk    PACKETDONE      ;is packet complete?
        bz      WIP_2           ;br if not
;
        call    LED3on
        lac     Duplex          ;see if in duplex mode
        bz      WIP_1           ;br if not
        call    Transmit_On     ;turn on xmitter        
        lac     TXDelay         ;start TXDelay timer
        sacl    Timer1
        call    Send_Flag       ;start sending flag bytes
        lalk    Wait_TX_Delay   ;set next state
        sacl    Hout_Vector
        b       WIP_2
WIP_1:
        lalk    Wait_No_Carrier ;set next state
        sacl    Hout_Vector
WIP_2:
        ret
;..............................................................................
;==============================================================================
;      =========       W a i t _ T X _ D e l a y ( )     ========
;==============================================================================
; This routine sends continuous flag bytes and waits for .01*TXDelay 
;Seconds then switches to the Send_HDLC_Data state.
;
;void Wait_TX_Delay( void ){
;   if( Flags.NEXTBYTE ){       // if ok to send next flag byte
;      Send_Flag();
;      if( !Timer1 )            // if TXDelay time has expired
;         Hout_Vector = Send_HDLC_Data;
;   }     
;}
;
Wait_TX_Delay:
        bit     Flags,T_NEXTBYTE
        bbz     WaitTXD_1
        call    Send_Flag       ;send flags while waiting
        lac     Timer1
        bnz     WaitTXD_1
        lalk    Send_HDLC_Data  ;set next state
        sacl    Hout_Vector
WaitTXD_1:
        ret
;..............................................................................
;==============================================================================
;      =========       W a i t _ N o _ C a r r i e r ( )     ========
;==============================================================================
; This routine waits for carrier detect to become inactive. When it
;becomes inactive, the 8 bit random number "Random0" is compared to the
;Persistence value and if Greater then the Wait_Slotime state is
;dispatched to wait .01*Slotime seconds.
;
;void Wait_No_Carrier( void )
;{
;   Make_random();
;   if( !CarDet ){
;      if( Random0 <=Persistence ){
;         Transmit_On();                // Turn on xmitter
;         Timer1 = TXDelay;             // start TXDelay timer
;         Send_Flag();                  // Start sending flags
;         Hout_Vector = Wait_TX_Delay;
;      }
;      else{
;         Timer1 = Slotime;             // start Slotime timer
;         Hout_Vector = Wait_Slotime;
;      }
;   }
;}
;
Wait_No_Carrier:
        call    Make_Random
        bit     Flags,T_CARRIERDET
        bbnz    WNC_1           ;br if is a carrier detected
;
        lac     Random0,4
        andk    0FFh,4          ;use middle 8 bits of Random0
        sub     Persistence,4
        bgz     WNC_2           ;br if( Random > Persistence )
;
        call    Transmit_On
        lac     TXDelay         ;start TXDelay timer
        sacl    Timer1
        call    Send_Flag
        lalk    Wait_TX_Delay   ;set next state
        sacl    Hout_Vector
        b       WNC_1
WNC_2:
        lac     Slotime         ;start Slotime timer
        sacl    Timer1
        lalk    Wait_Slotime    ;set next state
        sacl    Hout_Vector
WNC_1:
        ret
;..............................................................................
;==============================================================================
;      =========       W a i t _ S l o t i m e ( )     ========
;==============================================================================
; This routine waits for .01*Slotime Seconds then switches back to the
; Wait_No_Carrier state.
;
;void Wait_Slotime( void ){
;   if( !Timer1 )            // if Slotime timer has expired
;      Hout_Vector = Wait_No_Carier;
;}
;
Wait_Slotime:
        lac     Timer1
        bnz     Wait_S_1
        lalk    Wait_No_Carrier         ;set next state
        sacl    Hout_Vector
Wait_S_1:
        ret
;..............................................................................
;==============================================================================
;      =========       S e n d _ H D L C _ D a t a ( )     ========
;==============================================================================
;
;void Send_HDLC_Data()
;{
;   if( Flags.NEXTBYTE ){       // if ok to send new HDLC byte
;      Flags.NEXTBYTE = FALSE;
;      Flags.SENDFLAG = FALSE;
;      if( ++KSin_Tail >= KSin_Que+INQUESIZE-1 )
;         KSin_Tail = KSin_Que;
;      TX_NewByte = KSin_Que[ KSin_Tail ];
;      switch( TX_NewByte ){
;      case DATAVALID:          // keep sending data
;         break;
;      case PACKETDONE:
;         Send_Flag();          // separate next packet with flag byte 
;         break;
;      default:                 // start shutting down xmitter
;         Timer1 = TX_Tail;
;         Send_Flag();
;         Hout_Vector = Wait_TX_Tail;
;         break;
;      }
;   }
;}
;.  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
;                       Before a packet is complete
;                              KSin_Que[] 
;                          --------------------
;                         |         :          |
;                          --------------------
;    KSin_Tail--->        |    0    :    0     |  start of data packet
;                          --------------------
;                         |    1    :   D0     |  1st data byte
;                          --------------------
;                         |    1    :   D1     |  2nd data byte
;                          --------------------
;                         |    x    :    x     |  location for next data byte
;                          --------------------
;                         |         :          |
;                          --------------------
;
;.  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
;                        When a packet is complete
;                              KSin_Que[] 
;                          --------------------
;                         |         :          |
;                          --------------------
;    KSin_Tail--->        |    2    :    0     |  start of data packet
;        |                 --------------------
;        |                |    1    :   D0     |  1st data byte
;        |                 --------------------
;        |                |    1    :   D1     |  2nd data byte
;        |                 --------------------
;        |                |    1    :   D2     |  3rd and last data byte
;        |                 --------------------
;        V                |     0   :    0     |  End of packet
;                          --------------------
;
;
Send_HDLC_Data:
        bit     Flags,T_NEXTBYTE
        bbz     Send_HDLC_1             ;br  if( !Flags.NEXTBYTE )
        lac     Flags
        andk    ~(NEXTBYTE|SENDFLAG)    ;clr Flags.NEXTBYTE and Flags.SENDFLAG
        sacl    Flags
;
        larp    AR1             ;ARP = AR1
        lar     AR1,KSin_Tail
        mar     *+              ;++KSin_Tail
        lrlk    AR0,KSin_Que+INQUESIZE-1        ;check for wrap around
        cmpr    CMGT            ;TC= (AR1>AR0)
        bbz     SendHDLC_2      ;br if no wrap
        lrlk    AR1,KSin_Que    ;else wrap it around
SendHDLC_2:
        sar     AR1,KSin_Tail
        lac     *
        sacl    TX_NewByte      ;NewByte= KSin_Que[ KSin_Tail ];
        andk    DATAVALID       ;is packet complete?
        bnz     Send_HDLC_1     ;br if not end of packet
;
        lac     *
        andk    PACKETDONE      ;is next packet complete?
        bnz     Send_HDLC_3     ;br if so
        lac     TXtail          ;else start shutting down
        sacl    Timer1          ;start TXtail timer
        lalk    Wait_TX_Tail    ;set next state
        sacl    Hout_Vector
Send_HDLC_3:
        call    Send_Flag       ;separate packets with a flag byte
Send_HDLC_1:
        ret
;..............................................................................
;==============================================================================
;      =========       W a i t _ T X _ T a i l ( )     ========
;==============================================================================
; This routine sends continuous flag bytes and waits for .01*TX_Tail 
;Seconds then dispatches Wait_TX_Done state to wait for last flag byte to
;finish.
;
;void Wait_TX_Tail( void ){
;   if( Flags.NEXTBYTE ){       // if ok to send next flag byte
;      Send_Flag();
;      if( !Timer1 )            // if TX_Tail time has expired
;         Hout_Vector = Wait_TX_Done;
;   }     
;}
;
Wait_TX_Tail:
        bit     Flags,T_NEXTBYTE
        bbz     WaitTX_T_1
        call    Send_Flag       ;send flag bytes while waiting
        lac     Timer1
        bnz     WaitTX_T_1
        lalk    Wait_TX_Done    ;set next state
        sacl    Hout_Vector
WaitTX_T_1:
        ret
;..............................................................................
;==============================================================================
;      =========       W a i t _ T X _ D o n e ( )     ========
;==============================================================================
; This routine waits for the last flag byte to be sent then turns off
; the transmitter and returns to the Wait_In_Packet state.
;
;void Wait_TX_Done( void ){
;   if( Flags.NEXTBYTE ){       // if ok to send next flag byte
;      Transmit_Off()
;      Hout_Vector = Wait_In_Packet;
;   }     
;}
;
Wait_TX_Done:
        bit     Flags,T_NEXTBYTE
        bbz     WaitTX_D_1
        call    Transmit_Off    ;turn off xmitter
        lalk    Wait_In_Packet  ;set next state
        sacl    Hout_Vector
WaitTX_D_1:
        ret
;..............................................................................
;==============================================================================
;      =========          S e n d _ F l a g ( )     ========
;==============================================================================
; sets flags for low level output routines to ship out a flag byte
Send_Flag:
        lac     Flags
        ork     SENDFLAG
        andk    ~NEXTBYTE
        sacl    Flags
        lack    FLAGBYTE
        sacl    TX_NewByte
        ret
;..............................................................................
;==============================================================================
;      =========          T r a n s m i t _ O n ( )     ========
;==============================================================================
;       This routine turns on the transmitter.
;
Transmit_On:
        lac     Flags
        ork     TRANSMITON
        sacl    Flags
        call    LED2on
;
        bit     Flags,T_RADIO_PORT      ;check which port to use
        bbnz    Ton_1                   ;br if Port2
        call    PTT1_on
        ret
Ton_1:  call    PTT2_on
        ret
;..............................................................................
;==============================================================================
;      =========          T r a n s m i t _ O f f ( )     ========
;==============================================================================
;       This routine turns off the transmitter.
;
Transmit_Off:
        lac     Flags
        andk    ~TRANSMITON
        sacl    Flags
        call    LED2off
        bit     Flags,T_RADIO_PORT      ;check which port to use
        bbnz    Toff_1                  ;br if Port2
        call    PTT1_off
        ret
Toff_1: call    PTT2_off
        ret
;..............................................................................
;==============================================================================
;      =========      M a k e _ R a n d o m  ( )     ========
;==============================================================================
;       This routine generates a pseudo random number using linear
;congruential method ( RandNumber(n)=RandNumber(n-1)*K) + I )mod M.
; where RandNumber(n)=current random number
;       RandNumber(n-1) = previous random number
;       K = multiplier constant ( 31821 for this app )
;       I = Increment constant  ( 13849 for this app )
;       M = modulus ( 65536 for 16 bit 320C25 processor)
;
Make_Random:
        spm     0               ;set no shift of P
        rovm                    ;let overflo for modulus count
        lt      Random0
        mpy     Rand_K
        pac
        adlk    RAND_INC
        sacl    Random0
        sovm                    ;saturate on overflow
        spm     1               ;Shift P for Q15
        ret
;.....................E N D   o f   K S 1 2 H O U T . A S M ...................
