;******************************************************************************
;                      K S 1 2 K I N . A S M
;******************************************************************************
;   This module waits for a KISS format data byte to be received by the UART
;and places it in the circular buffer "KSin_Que" if it is KISS data ;or
;decodes the KISS command.
;
;==============================================================================
;      ========          I n i t _ K i n ( )     ==========
;==============================================================================
;void Init_Kin(void)
;{
;   if( TNC_INPUT.pin18 == +5V )
;      Flags.PTT1_LOGIC = TRUE;
;   else
;      Flags.PTT1_LOGIC = FALSE;
;   if( TNC_INPUT.pin16 == +5V )
;      Flags.PTT2_LOGIC = TRUE;
;   else
;      Flags.PTT2_LOGIC = FALSE;
;   PTT1_off();
;   PTT2_off();
;   Hardware = HARDWARE_INIT;
;   if( Hardware & 0x80 ){
;      Flags.RADIO_PORT = 1;
;      Sel_Input( REC_IN_2 );
;   }
;   else{
;      Flags.RADIO_PORT = 0;
;      Sel_Input( REC_IN_1 );
;   }
;   Sel_Gain( Hardware & 0x7F );
;}
Init_Kin:                       ;initialize variables
        in      Temp1,TNC_INPUT ;read PTT polarity jumper value
        lac     Temp1
        andk    RDO             ;chk PTT1
        bnz     In_K_1          ;br if port1 PTT is NEG logic
        lac     Flags           ;else is POS logic
        ork     PTT1_LOGIC
        b       In_K_2
In_K_1: lac     Flags
        andk    ~PTT1_LOGIC
        b       In_K_2
In_K_2: sacl    Flags
        lac     Temp1
        andk    XCLKO           ;chk PTT2
        bnz     In_K_3          ;br if port2 PTT is NEG logic
        lac     Flags           ;else is POS logic
        ork     PTT2_LOGIC
        b       In_K_4
In_K_3: lac     Flags
        andk    ~PTT2_LOGIC
In_K_4: sacl    Flags
;
        call    PTT1_off
        call    PTT2_off
;
        lack    HARDWARE_INIT   ;A/D gain and Radio port initialization
        sacl    Hardware
        bit     Hardware,7      ;bit 7 of Hardware byte selects radio port
        lac     Flags
        andk    ~RADIO_PORT
        sacl    Flags
        lack    REC_IN_1
        bbz     In_K_5
        lac     Flags
        ork     RADIO_PORT
        sacl    Flags
        lack    REC_IN_2
In_K_5: sacl    Temp0
        call    Sel_Input
;
        lac     Hardware        ;store gain part of Hardware byte
        andk    07Fh
        sacl    Temp0
        call    Sel_Gain
;
        lalk    TXDELAY_INIT*2          ;initialize KISS parameters
        sacl    TXDelay
        lalk    PERSISTENCE_INIT
        sacl    Persistence
        lalk    SLOTTIME_INIT*2
        sacl    Slotime
        lalk    TXTAIL_INIT*2
        sacl    TXtail
        lalk    DUPLEX_INIT
        sacl    Duplex  
;
        larp    AR0
        lrlk    AR0,KSin_Que
        sar     AR0,KSin_Head
        sar     AR0,KSin_Tail
        zac     
        sacl    *
;
        lalk    Kin_TopLevel    ;initialize state vector
        sacl    Kin_Vector
        ret
;..............................................................................
;========================================================================
;                 S e r v i c e _ K i n ( )
;========================================================================
;void Service_Kin()
:{
;   if( !UART_EMPTY ){       // See if anything in UART rceive buffer
;      Temp0 = UART INPUT BYTE;
;      AX0 = Temp0;
;      Kin_Vector();         // call state routines
;   }
;}
;
Service_Kin:
        lalk    UART_SEL_LSR
        sacl    Temp0
        out     Temp0,UART_CTRL
        in      Temp0,UART_READ
        lack    UART_LSR_RBF
        and     Temp0
        bz      SKi_1           ;br if UART receive buffer empty
        lalk    UART_SEL_RX
        sacl    Temp0
        out     Temp0,UART_CTRL
        in      Temp0,UART_READ  ;Temp0 = latest byte from UART
        lac     Temp0            ;just get lower byte
        andk    0FFh
        sacl    Temp0
        lar     AR0,Temp0
        lac     Kin_Vector
        cala                    ;call state vector    
SKi_1:
        ret
;........................................................................
;========================================================================
;                 K i n _ T o p L e v e l ( )
;========================================================================
; This routine decodes the top level KISS commands from the UART.
;Called with new data byte in Temp0 and in AX0 reg.
;void Kin_TopLevel( void )
;{
;   if(AR0 != FEND){          // if not another flag start or end
;      switch( AR0 & 0x0F ){  // get ready for second parameter byte
;         case TXDELAY_CMD:
;            Kin_Vector = Get_TXdelay;
;            break;
;         case P_CMD:
;            Kin_Vector = Get_Persistence;
;            break;
;         case SLOTIME_CMD:
;            Kin_Vector = Get_Slotime;
;            break;
;         case TXTAIL_CMD:
;            Kin_Vector = Get_TXtail;
;            break;
;         case DUPLEX_CMD:
;            Kin_Vector = Get_Duplex;
;            break;
;         case HARDWARE_CMD:
;            Kin_Vector = Get_Hardware;
;            break;
;         case DATAFRAME_CMD:
;            KSin_Start = KSin_Head;
;            Kin_Vector = Kin_DataLevel;
;            TX_crc = 0xFFFF;
;            break;
;         default:
;            Kin_Vector = Kin_TopLevel;
;            break;
;      }
;   }
;}
;.............................................................................
Kin_TopLevel:
        larp    AR1             ;ARP = AR1, AR0=Temp0=new data from UART
        lark    AR1,FEND
        cmpr    CMEQ
        bbz     Kin_1           ;br if AR0 != FEND
        ret
Kin_1:
        lac     Temp0
        andk    0Fh
        sacl    Temp0
        lar     AR0,Temp0       ;AR0 = AR0 & 0x0F strip off port ID
        lark    AR1,TXDELAY_CMD
        cmpr    CMEQ
        bbz     Kin_2           ;br if AR0 != TXDELAY_CMD
        lalk    Get_TXdelay     ;set next state
        sacl    Kin_Vector
        ret
Kin_2:
        lark    AR1,PERSIST_CMD
        cmpr    CMEQ
        bbz     Kin_3           ;br if AR0 != PERSIST_CMD
        lalk    Get_Persistence ;set next state
        sacl    Kin_Vector
        ret
Kin_3:
        lark    AR1,SLOTIME_CMD
        cmpr    CMEQ
        bbz     Kin_4           ;br if AR0 != SLOTIME_CMD
        lalk    Get_Slotime     ;set next state
        sacl    Kin_Vector
        ret
Kin_4:
        lark    AR1,TXTAIL_CMD
        cmpr    CMEQ
        bbz     Kin_5           ;br if AR0 != TXTAIL_CMD
        lalk    Get_TXtail      ;set next state
        sacl    Kin_Vector
        ret
Kin_5:
        lark    AR1,DUPLEX_CMD
        cmpr    CMEQ
        bbz     Kin_6           ;br if AR0 != DUPLEX_CMD
        lalk    Get_Duplex      ;set next state
        sacl    Kin_Vector
        ret
Kin_6:
        lark    AR1,HARDWARE_CMD
        cmpr    CMEQ
        bbz     Kin_7           ;br if AR0 != HARDWARE_CMD
        lalk    Get_Hardware    ;set next state
        sacl    Kin_Vector
        ret
Kin_7:
        lark    AR1,DATAFRAME_CMD
        cmpr    CMEQ
        bbz     Kin_TopExit     ;br if AR0 != DATAFRAME_CMD
;                               ;Begin new data packet
        lar     AR1,KSin_Head
        sar     AR1,KSin_Start  ; KSin_Start = KSin_Head;
;
        lalk    Kin_DataLevel   ;set next state
        sacl    Kin_Vector
        zac
        cmpl
        sacl    TX_crc          ;TC_crc = 0xFFFF
        ret
;.........................................................................
;========================================================================
;                 K i n _ D a t a L e v e l ( )
;========================================================================
; This routine places new KISS data from the UART into the KSin_Que and
;calculates a running CRC value on the data.
;Called with new data byte in Temp0 and in AX0 reg.
;
;void Kin_DataLevel( void )
;{
;   switch( AR0 ){
;      case FESC:                       // if a KISS escape code
;         Kin_Vector = ESC_Level;
;         break;
;      case FEND:                       // if a KISS end of packet code
;         Temp0 = ~TX_crc & 0xFFh;      // put TX_crc lsb in Que
;         Put_In_Que( Temp0 );
;         Temp0 = ~TX_crc >> 8;         // put TX_crc msb in Que
;         Put_In_Que( Temp0 );
;         Put_In_Que( 0 );              // point to start of next packet
;         KSin_Que[ KSin_Start ] = PACKETDONE; // signal packet done
;         Kin_Vector = Kin_TopLevel;
;         break;
;      default:                         //  else stuff it in KSin_Que
;         calc_TXcrc();
;         Put_In_Que( Temp0 );
;         break;
;   }
;}
;.  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
;                            Initial state
;                              KSin_Que[] 
;                          --------------------
;                         |         :          |
;                          --------------------
;    KSin_Start--->       |    0    :    0     |  start of data packet
;    KSin_Head             --------------------
;                         |    x    :    x     | 
;                          --------------------
;                         |    x    :    x     | 
;                          --------------------
;                         |    x    :    x     | 
;                          --------------------
;                         |         :          |
;                          --------------------
;
;.  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
;                        Before a packet is complete
;                              KSin_Que[] 
;                          --------------------
;                         |         :          |
;                          --------------------
;    KSin_Start--->       |    0    :    0     |  start of data packet
;                          --------------------
;                         |    1    :   D0     |  1st data byte
;                          --------------------
;    KSin_Head--->        |    1    :   D1     |  2nd data byte
;                          --------------------
;                         |    x    :    x     |  location for next data byte
;                          --------------------
;                         |         :          |
;                          --------------------
;
;                         When a packet is complete
;                              KSin_Que[] 
;                          --------------------
;                         |         :          |
;                          --------------------
;   KSin_Start--->        |    2    :    0     |  start of data packet
;                           --------------------
;                         |    1    :   D0     |  1st data byte
;                          --------------------
;                         |    1    :   D1     |  2nd data byte
;                          --------------------
;                         |    1    :   D2     |  3rd and last data byte
;                          --------------------
;    KSin_Head--->        |    0    :    0     |  End of packet
;                           --------------------
;
;........................................................................
Kin_DataLevel:
        larp    AR1             ;ARP = AR1, AR0=Temp0=new data from UART
        lark    AR1,FESC
        cmpr    CMEQ
        bbz     Kin_D1          ;br if AR0 != FESC
        lalk    ESC_Level       ;set next state to process ESC commands
        sacl    Kin_Vector
        ret
;
Kin_D1: lark    AR1,FEND
        cmpr    CMEQ
        bbz     Kin_D2          ;br if AR0 != FEND
        lac     TX_crc          ;here if data is FEND of packet
        cmpl
        sacl    TX_crc          ;TX_crc = ~TX_crc
        andk    0FFh
        sacl    Temp0           ;Temp0 = TX_crc & 0x00FF;
        call    Put_In_Que
        zac
        rsxm                    
        lac     TX_crc,8
        sach    Temp0           ;Temp0 = TX_crc >> 8;
        ssxm
        call    Put_In_Que
;         
        call    Put_In_Que      ;point to start of next packet
        zac                     ;zero start location of next
        sacl    *               ;cheat by storing a zero at (AR1)
                                ;KSin_Que[ KSin_Head ] = 0;
;
        larp    AR1             ;ARP = AR1
        lar     AR1,KSin_Start  ;signal packet done
        lalk    PACKETDONE      ;KSin_Que[ KSin_Start ] = PACKETDONE
        sacl    *
;
        b       Kin_TopExit
;
Kin_D2:                         ;calc crc and put data in que
        call    Calc_TXcrc
        call    Put_In_Que
        ret
;.......................................................................
;========================================================================
;                 E S C _ L e v e l ( )
;========================================================================
; This routine places FEND or FESC data into the KSin_Que when the
;KISS FESC sequence is received.
;Called with new data byte in Temp0 and in AX0 reg.
;
;void ESC_Level()
;{
;   switch( AR0 ){
;      case TFEND:              // if a KISS escape FEND code
;         Temp0 = FEND;
;         calc_TXcrc();
;         Put_In_Que( Temp0 );
;         break;
;      case TFESC:              // if a KISS escape FESC code
;         Temp0 = FESC;
;         calc_TXcrc();
;         Put_In_Que( Temp0 );
;         break;
;      default:
;         break;
;   }
;   Kin_Vector = Kin_DataLevel;
;}
;........................................................................
ESC_Level:
        larp    AR1             ;ARP = AR1, AR0=Temp0=new data from UART
        lark    AR1,TFEND
        cmpr    CMEQ
        bbz     Kin_E1          ;br if AR0 != TFEND
        lack    FEND
        sacl    Temp0
        b       Kin_E2
Kin_E1:
        lark    AR1,TFESC
        cmpr    CMEQ
        bbz     Kin_E4          ;br if AR0 != TFESC
        lack    FESC
        sacl    Temp0
;
Kin_E2: call    Calc_TXcrc
        call    Put_In_Que
;
Kin_E4: lalk    Kin_DataLevel   ;set next state
        sacl    Kin_Vector
        ret
;.........................................................................
;................   KISS  Command data storage stages   ..................        
;.........................................................................
Get_TXdelay:
        lac     Temp0,1         ;mult by 2 since timer is 5Msecs not 10
        sacl    TXDelay
        b       Kin_TopExit
Get_Persistence:
        lac     Temp0,1         ;mult by 2 since timer is 5Msecs not 10
        sacl    Persistence
        b       Kin_TopExit
Get_Slotime:
        lac     Temp0,1         ;mult by 2 since timer is 5Msecs not 10
        sacl    Slotime
        b       Kin_TopExit
Get_TXtail:
        lac     Temp0,1         ;mult by 2 since timer is 5Msecs not 10
        sacl    TXtail
        b       Kin_TopExit
Get_Duplex:
        lac     Temp0
        sacl    Duplex
        b       Kin_TopExit
Get_Hardware:
        lac     Temp0
        sacl    Hardware
        bit     Hardware,7      ;bit 7 of Hardware byte selects radio port
        lac     Flags
        andk    ~RADIO_PORT
        sacl    Flags
        lack    REC_IN_1
        bbz     GH_1
        lac     Flags
        ork     RADIO_PORT
        sacl    Flags
        lack    REC_IN_2
GH_1:   sacl    Temp0
        call    Sel_Input
;
        lac     Hardware        ;store gain part of Hardware byte
        andk    07Fh
        sacl    Temp0
        call    Sel_Gain
        b       Kin_TopExit
Kin_TopExit:
        lalk    Kin_TopLevel    ;set next state to top level decode
        sacl    Kin_Vector
        ret
;
;.......................................................................
;========================================================================
;                 C a l c _ T X c r c ( )
;========================================================================
; This routine calculates the next transmit CRC value ( TX_crc )from the
;data stored in Temp0.
;
;void Calc_TXcrc( Temp0 )
;{
;   crc_Temp = Temp0 ^ TX_crc & 0xFF;
;   TX_crc >>= 8;
;   TX_crc &= 0x00FF;
;   TX_crc ^= CRC_TABLE[ crc_Temp ];
;}
;.......................................................................
Calc_TXcrc:
        lac     Temp0           ;   crc_temp = Temp0 ^ TX_crc & 0xFF;
        xor     TX_crc
        andk    0FFh
        sacl    crc_temp
;
        zac                     ;   TX_crc >>= 8;
        rsxm                    ;   TX_crc &= 0x00FF;
        lac     TX_crc,8
        sach    TX_crc
        ssxm
;
        lac     crc_temp        ;   TX_crc ^= CRC_TABLE[ crc_temp ];
        adlk    CRC_TABLE
        tblr    crc_temp
        zals    crc_temp
        xor     TX_crc
        sacl    TX_crc
        ret
;......................................................................
;========================================================================
;                    P u t _ I n _ Q u e ( Temp0 )
;========================================================================
; This routine places byte located in Temp0 into the KSin_Que. 
;DATAVALID bit (bit 15) is set as well.
;
;void Put_In_Que(){
;   if( ++KSin_Head > KSin_Que+INQUESIZE-1 )
;      KSin_Head = KSin_Que;
;   KSin_Que[ KSin_Head ] = Temp0 | DATAVALID;
;}
;
Put_In_Que:
        larp    AR1             ;ARP = AR1, AR0=Temp0=new data from UART
        lar     AR1,KSin_Head
        mar     *+              ;++KSin_Head
        lrlk    AR0,KSin_Que+INQUESIZE-1        ;check for wrap around
        cmpr    CMGT            ;TC = (AR1>AR0)
        bbz     PiQ_1           ;br if no wrap
        lrlk    AR1,KSin_Que    ;else wrap it around
PiQ_1:  sar     AR1,KSin_Head
        lac     Temp0
        ork     DATAVALID
        sacl    *               ;KSin_Que[ KSin_Head ] = Temp0 | DATAVALID
        ret
;---------------------------------------------------------------------
Outaccl:
        ldpk    8
        sacl    PRVAL_MEM
        call    PRVAL16
        call    SD_CRLF
        ret

;        lac     Temp0
;        call    Outaccl
;.....................E N D   o f   K S 1 2 K I N . A S M ....................
