;******************************************************************************
;                      K S 1 2 K O U T . A S M
;******************************************************************************
;   This module waits for a received packet to be placed in the circular
;buffer "KISS_Out_Que" then transmits the bytes in the packet out the 
;UART channel in KISS format.
;
;==============================================================================
;     ========          I n i t _ K o u t ( )     ==========
;==============================================================================
Init_Kout:                              ;initialize variables
        larp    AR0
        lrlk    AR0,KSout_Que
        sar     AR0,KSout_Head
        sar     AR0,KSout_Tail
        zac     
        sacl    *
        lalk    Wait_Out_Packet         ;set next state
        sacl    Kout_Vector
        ret
;..............................................................................
;==============================================================================
;                 S e r v i c e _ K o u t ( )
;==============================================================================
Service_Kout:
        lalk    UART_SEL_LSR    ;select line status register
        sacl    Temp0
        out     Temp0,UART_CTRL
        in      Temp0,UART_READ ;read line status register
        lack    UART_LSR_THRE
        and     Temp0           ;check  xmit holding register full bit
        bz      SKo_1           ;br if UART Transmit buffer full
;
        lac     Kout_Vector     ;call state vector if OK to send to UART
        cala
SKo_1:  ret

;-----------------------------------------------------------------------------
;..............................................................................
;==============================================================================
;     ========     W a i t _ O u t _ P a c k e t ( )       ==========
;==============================================================================
; This routine checks the KSout_Que for a new packet then dispatches
; the next state for converting and sending a KISS format packet to
;the UART 
;
;void Wait_Out_Packet( void )
;{
;   if( KSout_Que[ KSout_Tail ] & PACKETDONE ){ // if packet complete 
;      Kout_Vector = Wait_TX_Delay;
;   }
;}
;.............................................................................
Wait_Out_Packet:
        larp    AR1             ;ARP = AR1
        lar     AR1,KSout_Tail
        lac     *               ;acc = KSout_Que[ KSout_Tail ]
        andk    PACKETDONE      ;is packet complete?
        bz      WOP_1           ;br if not
        lack    FEND            ;send KISS start character
        sacl    Temp0
        call    Send_To_UART
        lalk    Send_Dataframe  ;set next state
        sacl    Kout_Vector
        call    LED5on
WOP_1:  ret
;..............................................................................

;==============================================================================
;     ========     S e n d _ B u f f e r ( )       ==========
;==============================================================================
; This routine sends a received data packet to the KISS UART.
;void Send_Buffer(void)
;{
;   if( ++KSout_Tail >= KSout_Que+OUTQUESIZE-1 )  // handle wrap around
;      KSout_Tail = KSout_Que;
;   Temp1 = KSout_Que[ KSout_Tail ];    // Temp1 = next data byte
;   if( Temp1 & DATAVALID ){
;      switch( Temp1& 0FFh ){
;         case FEND:
;            Temp0 = FESC;
;            Kout_Vector = ESC_FEND_State;
;            break;
;         case FESC:
;            Temp0 = FESC;
;            Kout_Vector = ESC_FESC_State;
;            break;
;         default:
;            Temp0 = Temp1;
;            break;
;      }
;   }
;   else{               // is end of data packet
;      Temp0 = FEND;
;      Kout_Vector = Wait_Out_Packet;
;   }
;   Send_To_UART(Temp0);
;}
;
Send_Buffer:
        larp    AR1             ;ARP = AR1
        lar     AR1,KSout_Tail
        mar     *+              ;++KSout_Tail
        lrlk    AR0,KSout_Que+OUTQUESIZE-1  ;check for wrap around
        cmpr    CMGT            ;TC = (AR1>AR0)
        bbz     SB_1            ;br if no wrap
        lrlk    AR1,KSout_Que   ;else wrap it around
SB_1:
        sar     AR1,KSout_Tail
        lac     *
        sacl    Temp1           ;Temp1 = KSout_Que[ KSout_Tail ]
        andk    DATAVALID
        bz      SB_2            ;br if end of packet
;
        lac     Temp1
        andk    0FFh
        sacl    Temp1           ;strip off just the lower byte
;
        lar     AR0,Temp1       ;AR0 = newdata byte
        lark    AR1,FEND
        cmpr    CMEQ
        bbz     SB_3            ;br if AR0 != FEND
        lalk    ESC_FEND_State  ;set next state
        b       SB_5
SB_3:
        lark    AR1,FESC
        cmpr    CMEQ
        bbz     SB_4            ;br if AR0 != FESC
        lalk    ESC_FESC_State  ;set next state
        b       SB_5
SB_4:
        lac     Temp1
        sacl    Temp0
        b       SB_6
;
SB_5:   sacl    Kout_Vector
        lack    FESC            ;send KISS FESC character
        sacl    Temp0
        b       SB_6
;
SB_2:   lalk    Wait_Out_Packet ;set next state
        sacl    Kout_Vector
        call    LED5off
        lack    FEND            ;send KISS FEND character
        sacl    Temp0
SB_6:   call    Send_To_UART
        ret
;.............................................................................
;==============================================================================
;     ========     S e n d _ D a t a f r a m e ( )       ==========
;==============================================================================
; This routine sends a DATA_FRAME_CMD to the UART.
;
Send_Dataframe:
        lack    DATAFRAME_CMD   ;send KISS DATAFRAME character
        sacl    Temp0
        call    Send_To_UART
        lalk    Send_Buffer     ;set next state
        sacl    Kout_Vector
        ret
;.............................................................................
;==============================================================================
;     ========     E S C _ F E N D _ S t a t e ( )       ==========
;==============================================================================
; This routine sends a TFEND to the UART.
;
ESC_FEND_State:
        lack    TFEND           ;send KISS TFEND character
        sacl    Temp0
        call    Send_To_UART
        lalk    Send_Buffer     ;set next state
        sacl    Kout_Vector
        ret
;.............................................................................
;==============================================================================
;     ========     E S C _ F E S C _ S t a t e ( )       ==========
;==============================================================================
; This routine sends a TFESC to the UART.
;
ESC_FESC_State:
        lack    TFESC           ;send KISS TFESC character
        sacl    Temp0
        call    Send_To_UART
        lalk    Send_Buffer     ;set next state
        sacl    Kout_Vector
        ret
;.............................................................................
;==============================================================================
;     ========     S e n d _ T o _ U A R T ( )       ==========
;==============================================================================
;       This routine sends a byte in Temp0 out the KISS interface UART.
;
Send_To_UART:
        lalk    UART_SEL_RX     ;select transmit holding register
        sacl    Temp1
        out     Temp1,UART_CTRL
        out     Temp0,UART_WRITE
        ret
;...................E N D   o f   K S 1 2 K O U T. A S M ......................
