;******************************************************************************
;    * * * * * * *       K S 1 2 M A I N . A S M      * * * * * * * * 
;******************************************************************************
;       A KISS mode 1200 Baud AFSK Packet (Bell 202) modem for the DSP-93.
; Created       11-02-96        Moe Wheatley AE4JY
; Last Modified 12-20-96
;
;                 Assembly Instructions
;invoking:  
;tasm -3225 -lal -g0 ks12main.asm
;
;NOTE: The following source files are needed to assemble this code-
;  "ks12main.asm"
;  "ks12ain.asm"
;  "ks12aout.asm"
;  "ks12hin.asm"
;  "ks12hout.asm"
;  "ks12kin.asm"
;  "ks12kout.asm"
;  "ks12data.tbl"
;
;The following include files from TAPR are needed to assemble this code-
;  "REGS.INC"
;  "PORTS.INC"
;  "MONITOR.INC"
;  "SERIAL.INC"
;
;                     L E D   U S E A G E
;  LED211 is Monitor Active
;  LED212 is Transmitter ON
;  LED213 is Transmit Pending
;  LED214 is /DCD carrier detect
;  LED215 is Packet Received
;  LED216 is Input Level Indicator
;  LED217 is DCD carrier detect
;  LED218 is power
;
; License and Disclaimer
;  
; A no cost, non-exclusive license to use the enclosed software is granted
; to all users of the TAPR/AMSAT DSP-93 DSP computer, with the restriction
; that it be used only for non-commercial amateur radio applications. ALL
; other uses are prohibited under this license. Copyright 1996.
; 
; This software is EXPERIMENTAL. It is offered AS IS, without Warranty or
; Guarantee of any kind. NO representation is made that this software is
; suitable for any purpose, or that its operation is reliable. Use this software
; strictly at your own risk. The author, AMSAT and TAPR accept no liability
; for any loss or injury arising directly or indirectly from its use.
; 
; Never use this software under circumstances where its failure to operate
; or its failure to operate as expected, or the failure of the attendant
; hardware could create a dangerous situation, cause loss of data, or could
; cause damage to your amateur radio equipment, computer equipment, or
; any other equipment or property.
; 
; Please contact TAPR or the author for information on commercial licenses
; of this software.
; 
;------------------------------------------------------------------------------
;                     I N C L U D E   F I L E S
;------------------------------------------------------------------------------
        .nolist
#include  "REGS.INC"
#include  "PORTS.INC"
#include  "MONITOR.INC"
#include  "SERIAL.INC"
;        .list
        .nolist
;------------------------------------------------------------------------------
;                     M A C R O    D E F I N E S
;------------------------------------------------------------------------------
#define INC(xx) lac xx\ addk 1\ sacl xx
#define DEC(xy) lac xy\ subk 1\ sacl xy

;------------------------------------------------------------------------------
;            A R x    R e g i s t e r  U s e a g e
;------------------------------------------------------------------------------
;AR0 = general purpose temp pointer
;AR1 = general purpose temp pointer
;AR2 = general purpose temp pointer
;AR3 = Software Stack Pointer
;AR4 = Sample output que HEAD pointer (where data can be put for output)
;AR5 = Sample input que TAIL pointer (where data is taken for output and
;                                     input data placed during AIO irq )
;AR6 = Sample input que HEAD pointer ( where data can be taken out for input)
;AR7 = general purpose temp pointer
;------------------------------------------------------------------------------
;                     C O N S T A N T    E Q U A T E S
;------------------------------------------------------------------------------
TM_EN   .equ    008h    ;timer interrupt enable bit
RX_EN   .equ    010h    ;serial port recv enable bit
TX_EN   .equ    020h    ;serial port xmit enable bit
CMEQ    .equ    0       ;cmpr instruction EQUALTO operator
CMLT    .equ    1       ;cmpr instruction LESSTHAN operator
CMGT    .equ    2       ;cmpr instruction GREATERTHAN operator
CMNE    .equ    3       ;cmpr instruction NOTEQUAL operator
;                               ;sample rate = 5MHz/(RA*RB)
CMDA    .equ    CMDA_VAL(17)    ;AIO load RA/TA = 17 for SCF @ 294.1 kHz
CMDB    .equ    CMDB_VAL(27)    ;AIO load RB/TB = 27 for 10893.25 samples/sec
;  AIO configuration: HPF in, Sync, 3 V full scale, sin x/x
CMDC    .equ    AIO_CONFIG|AIO_INSADHPF|AIO_SYNC|AIO_GAIND61|AIO_GAIND70|AIO_INSSINX
;
;       Output tones frequency increment = 10893.25/32768 = .3324355 Hz
INCL            .equ    3610            ;incr value for 1.2 kHz
INCH            .equ    6618            ;incr value for 2.2 kHz
;       1200 BPS rate frequency increment = 10893.25/65536 = .166217745 Hz
INC1200         .equ    7219            ;incr value for 1200 BPS
;
MS5PERIOD       .equ    50000           ;Period for 5 Millisecond timer
                                        ; @ 10 MHZ   clock rate
;
;       KISS UART Baud rate constants
BAUD2400        .equ    128             ;4.9152MHz/(2400*16)
BAUD4800        .equ    64              ;4.9152MHz/(4800*16)
BAUD9600        .equ    32              ;4.9152MHz/(9600*16)
BAUD19200       .equ    16              ;4.9152MHz/(19200*16)
;
;****   change BAUDRATE here for desired KISS UART speed   *********
BAUDRATE        .equ    BAUD9600
;
RADIO_1         .equ    0               ;Radio port 1
RADIO_2         .equ    080h            ;Radio port 2
;GAIN0                                  ;gain = 1
;GAIN1                                  ;gain = 2
;GAIN2                                  ;gain = 4
;GAIN3                                  ;gain = 8
;GAIN4                                  ;gain = 16
;GAIN5                                  ;gain = 32
;GAIN6                                  ;gain = 64
;*******   change Initial Radio port and GAIN here    ***************
HARDWARE_INIT           .equ    GAIN4|RADIO_1
TXDELAY_INIT            .equ    50
PERSISTENCE_INIT        .equ    63
SLOTTIME_INIT           .equ    10
TXTAIL_INIT             .equ    5
DUPLEX_INIT             .equ    0
;
;       KISS Command Definitions
DATAFRAME_CMD   .equ    0
TXDELAY_CMD     .equ    1
PERSIST_CMD     .equ    2
SLOTIME_CMD     .equ    3
TXTAIL_CMD      .equ    4
DUPLEX_CMD      .equ    5
HARDWARE_CMD    .equ    6
;
;       KISS protocol constants 
FEND            .equ    0C0h
FESC            .equ    0DBh
TFEND           .equ    0DCh
TFESC           .equ    0DDh
FLAGBYTE        .equ    07Eh

;       Queing control flag definitions
DATAVALID       .equ    0100h           ;upper byte flag for valid data
PACKETDONE      .equ    0200h           ;upper byte flag for packet complete
;
;       Random number generator constants
RAND_K          .equ    31821
RAND_INC        .equ    13849
RAND_SEED       .equ    21845
;
;       Phase delay discriminator constant
DELAY_CONST     .equ    22615  ;calculated for group delay
;DELAY_CONST     .equ    23962  ;calculated for phase delay
;
;       misc receiver constants
SLICER_HYST     .equ    5       ;discriminator hysterysis
CAR_THRESHOLD   .equ    20      ;Carrier detect threshold
DCD_TIMEOUT     .equ    100     ;Carrier detect timeout limit
BITSINBYTE      .equ    8       ;restating the obvious
MAX_ONES        .equ    7       ;max number of received ones allowed
HDLC_FLAG       .equ    07Eh    ;HDLC flag byte value
GOOD_CRC        .equ    0F0B8h  ;good checksum value for packets
ADLIMIT         .equ    6280    ;Receive signal peak indicator limit
                                ;2^13*2.3/3 == 2.3 volt Peaks
ONESHOT_LEN     .equ    20      ;signal peak indicator display time
                                ;20 * .005 = .1 SEC
;
;------------------------------------------------------------------------------
;         M E M O R Y   P A G E   P O I N T E R    E Q U A T E S
;------------------------------------------------------------------------------
USER_ADR        .equ    USER_PAGE<<7
;
;------------------------------------------------------------------------------
;         I N T E R N A L   M E M O R Y    E Q U A T E S
;------------------------------------------------------------------------------
;B0 range is 200h to 2FFh       used for filter coefficients
;       (when addressed as program memory, 0FF00h to 0FFFF)
B0DADDR         .equ    0200h
B0PADDR         .equ    0FF00h
;
BPFIRCOEF       .equ    B0DADDR
BPFIR_LENGTH    .equ    50
LPFIRCOEF       .equ    BPFIRCOEF+BPFIR_LENGTH
LPFIR_LENGTH    .equ    32
; . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
;B1 range is 300h to 3FFh       used for filter delay lines
BPFdelay        .equ    300h
LPFdelay        .equ    BPFdelay+BPFIR_LENGTH+1
; . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
;B2 range is 060h to 07Fh       used for software stack and fast variables
;
ToutBuf         .equ    060h
;
EndStack        .equ    070h
Stack           .equ    07Fh    ;Stack grows down from 7F to 70;
;------------------------------------------------------------------------------
;               E X T E R N A L   M E M O R Y    E Q U A T E S
;------------------------------------------------------------------------------
;Variables in external memory addressed by direct mode (400h to 4FFh)
Temp0           .equ    000h
Temp1           .equ    001h
Port6mirror     .equ    002h
Port7mirror     .equ    003h
PortBmirror     .equ    004h
PhaseAcc        .equ    005h
PhaseInc        .equ    006h
Xmit_Phz_Acc    .equ    007h
OutBuf          .equ    008h
TX_OnesCounter  .equ    009h
TX_BitCounter   .equ    00Ah
TX_ByteRegister .equ    00Bh
TX_NewByte      .equ    00Ch
TX_NextByte     .equ    00Dh
TestOut         .equ    00Eh
Dely0           .equ    00Fh
Dely1           .equ    010h
DelyK           .equ    011h
Recv_Phz_Acc    .equ    012h
DCD_Counter     .equ    013h
DCD_OneShot     .equ    014h
;
Timer1          .equ    015h
Timer2          .equ    016h
Kin_Vector      .equ    017h
Kout_Vector     .equ    018h
Hin_Vector      .equ    019h
Hout_Vector     .equ    01Ah
TXDelay         .equ    01Bh
Persistence     .equ    01Ch
Slotime         .equ    01Dh
TXtail          .equ    01Eh
Duplex          .equ    01Fh
Hardware        .equ    020h
;
KSin_Tail       .equ    021h
KSin_Start      .equ    022h
KSin_Head       .equ    023h
KSout_Tail      .equ    024h
KSout_Start     .equ    025h
KSout_Head      .equ    026h
TX_crc          .equ    027h
RX_crc          .equ    028h
crc_temp        .equ    029h
Random0         .equ    02Ah
Rand_K          .equ    02Bh
;
RX_OnesCounter  .equ    02Ch
RX_BitCounter   .equ    02Dh
RX_ByteRegister .equ    02Eh
RX_NewByte      .equ    02Fh
GoodCRC         .equ    030h
AD_Limit        .equ    031h
;
Phz_Inc_1200    .equ    032h
Position_Sum    .equ    033h
Position_Ave    .equ    034h
Perftimer       .equ    035h
; . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
;               B I T    V A R I A B L E S
; . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
Flags           .equ    040h    ;bit variables
FLAGING         .equ    1<<0    ;xmiting a flag byte
T_FLAGING       .equ    0       ;test bit
TXLASTBIT       .equ    1<<1    ;state of previous output bit
T_TXLASTBIT     .equ    1       ;test bit
NEXTBYTE        .equ    1<<2    ;indicates next byte can be xmitted
T_NEXTBYTE      .equ    2       ;test bit
SENDFLAG        .equ    1<<3    ;indicates next byte to be sent is a flag
T_SENDFLAG      .equ    3       ;test bit
TRANSMITON      .equ    1<<4    ;indicates transmitter is on
T_TRANSMITON    .equ    4       ;test bit
TXPRESENTBIT    .equ    1<<5    ;present output MARK/SPACE state
T_TXPRESENTBIT  .equ    5       ;test bit
CARRIERDET      .equ    1<<6    ;carrier is detected
T_CARRIERDET    .equ    6       ;test bit
INPUTBIT        .equ    1<<7    ;present state of input data
T_INPUTBIT      .equ    7       ;test bit
NEWBYTERDY      .equ    1<<8    ;indicates a new byte has been received
T_NEWBYTERDY    .equ    8       ;test bit
NEWFLAGRDY      .equ    1<<9    ;indicates a new flag byte has been received
T_NEWFLAGRDY    .equ    9       ;test bit
RXLASTBIT       .equ    1<<10   ;state of previous input bit
T_RXLASTBIT     .equ    10      ;test bit
RADIO_PORT      .equ    1<<11   ;selects radio port(0=port1, 1=port2)
T_RADIO_PORT    .equ    11      ;test bit
PTT1_LOGIC      .equ    1<<12   ;selects PTT port1 logic sense(0=NEG, 1=POS)
T_PTT1_LOGIC    .equ    12      ;test bit
PTT2_LOGIC      .equ    1<<13   ;selects PTT port2 logic sense(0=NEG, 1=POS)
T_PTT2_LOGIC    .equ    13      ;test bit
PEAK_DET        .equ    1<<14   ;Peak Detect latch(1=A/D peak detected)
T_PEAK_DET      .equ    14      ;test bit
;
;Variables in external memory addressed by indirect mode (500h to 7FFFh)
SAMPQUESIZE     .equ    8
INQUESIZE       .equ    2500
OUTQUESIZE      .equ    2500
;                
Sample_Que:     .org    0500h
KSin_Que:       .org    $+SAMPQUESIZE
KSout_Que:      .org    $+INQUESIZE
                .org    $+OUTQUESIZE
;
;==============================================================================
;     ==========   I N T E R R U P T    V E C T O R S   ==========
;==============================================================================
        .org    1000h
        b       TimerIntService ;branch to timer interrupt service (TINT)
        b       RxIntService    ;SIO receive interrupt service routine
        b       TxIntService    ;SIO transmit interrupt service routine
        b       Start           ;branch to program start (TRAP)
        b       Start           ;branch to program start (RESET Vector)

;+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
;         E X T E R N A L     M O D U L E     I N C L U D E S 
;+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
#include  "ks12data.tbl"        ;Include constant tables
#include  "ks12ain.asm"
#include  "ks12aout.asm"
#include  "ks12hin.asm"
#include  "ks12hout.asm"
#include  "ks12kin.asm"
#include  "ks12kout.asm"
;
;==============================================================================
;     ==========   P R O G R A M   E N T R Y   P O I N T   ==========
;==============================================================================
Start:  
        dint                    ;disable interrupts
        rptk    7-1             ;free up hardware stack
        pop
        call    Init_System     ;Initialize the system
;
;==============================================================================
;       ==========    M A I N   P R O G R A M   L O O P     ==========
;==============================================================================
;       Poor man's multitasking Round Robin calling of all the tasks.
MainLoop:
;
;$$$$
;        call    FREQ_UP_1off
;        lac     Perftimer      ;this test code was used to measure
;        subk    1              ;the processing time through all the
;        sacl    Perftimer      ;service routines.
;        bnz     ML_1
;        lalk    500
;        sacl    Perftimer
;        call    FREQ_UP_1on        
;ML_1:
;$$$$
        call    Service_Ain     ;service input samples from A/D Que
;
        call    Service_Aout    ;service output samples to D/A Que
;
        call    Service_Hin     ;service HDLC input data
;
        call    Service_Hout    ;service HDLC output data
;
        call    Service_Kin     ;service KISS format data from UART
;
        call    Service_Kout    ;service KISS format data going to UART
;
        b       MainLoop        ;loop forever
;
;==============================================================================
;       ==========    I / O   S U B R O U T I N E S     ==========
;==============================================================================
;
;................     PORT 6 I/O routines     .................................
RD_Ion: lac     Port6mirror
        ork     RDI             ;RD_I bit
        b       Port6_Write
RD_Ioff:
        lac     Port6mirror
        andk    ~RDI            ;RD_I bit
        b       Port6_Write
RD_Clkon:
        lac     Port6mirror
        ork     RCLKI           ;RCLKI bit
        b       Port6_Write
RD_Clkoff:
        lac     Port6mirror
        andk    ~RCLKI          ;RCLKI bit
        b       Port6_Write
LED5on: lac     Port6mirror
        ork     LED_205         ;LED 205 bit
        b       Port6_Write
LED5off:
        lac     Port6mirror
        andk    LED_205_OFF
        b       Port6_Write
LED6on: lac     Port6mirror
        ork     LED_206         ;LED 206 bit
        b       Port6_Write
LED6off:
        lac     Port6mirror
        andk    LED_206_OFF
        b       Port6_Write
LED7on: lac     Port6mirror
        ork     LED_207         ;LED 207 bit
        b       Port6_Write
LED7off:
        lac     Port6mirror
        andk    LED_207_OFF
Port6_Write:
        sacl    Port6mirror
        out     Port6mirror,TNC_OUTPUT
        ret
;................     PORT 7 I/O routines     .................................
AIO_Reset:
        lac     Port7mirror
        ork     AIO_RESET               ;reset D15 to disconnect AIO (DX&DR)
        b       Port7_Write
AIO_Run:
        lac     Port7mirror
        andk    ~AIO_RESET              ;bring AIO out of reset
        b       Port7_Write
AIO_Enable:
        lac     Port7mirror
        ork     AIO_ENABLE
        b       Port7_Write
AIO_Disable:
        lac     Port7mirror
        andk    ~AIO_ENABLE
        b       Port7_Write
Sel_Input:                              ;Temp0 contains selection--
        lac     Port7mirror             ;(see PORTS.INC for definitions)
        andk    0FFF8h                  ;clear select bits
        or      Temp0
        b       Port7_Write
Sel_Gain:                               ;Temp0 contains selection--
        lac     Port7mirror             ;(see PORTS.INC for definitions)
        andk    0FFC7h                  ;clear gain bits
        or      Temp0
        b       Port7_Write
LED1on: lac     Port7mirror
        ork     LED_201                 ;LED 201 bit
        b       Port7_Write
LED1off:
        lac     Port7mirror
        andk    LED_201_OFF
        b       Port7_Write
LED2on: lac     Port7mirror
        ork     LED_202                 ;LED 202 bit
        b       Port7_Write
LED2off:
        lac     Port7mirror
        andk    LED_202_OFF
Port7_Write:
        sacl    Port7mirror
        out     Port7mirror,RADIO_GAIN
        ret
;
;................     PORT B I/O routines     .................................
PTT1_on:
        lac     PortBmirror             ;PTT radio port 1
        bit     Flags,T_PTT1_LOGIC      ;check logic sense
        bbz     PTT1_0                  ;br if NEG logic sense
        andk    ~PTT_R1
        b       PortB_Write
PTT1_0: ork     PTT_R1
        b       PortB_Write
PTT1_off:
        lac     PortBmirror             ;PTT radio port 1
        bit     Flags,T_PTT1_LOGIC      ;check logic sense
        bbz     PTT1_1                  ;br if NEG logic sense
        ork     PTT_R1
        b       PortB_Write
PTT1_1: andk    ~PTT_R1
        b       PortB_Write
;
PTT2_on:
        lac     PortBmirror             ;PTT radio port 2
        bit     Flags,T_PTT2_LOGIC      ;check logic sense
        bbz     PTT2_0                  ;br if NEG logic sense
        andk    ~PTT_R2
        b       PortB_Write
PTT2_0: ork     PTT_R2
        b       PortB_Write
PTT2_off:
        lac     PortBmirror             ;PTT radio port 2
        bit     Flags,T_PTT2_LOGIC      ;check logic sense
        bbz     PTT2_1                  ;br if NEG logic sense
        ork     PTT_R2
        b       PortB_Write
PTT2_1: andk    ~PTT_R2
        b       PortB_Write
;
LED3on: lac     PortBmirror
        ork     LED_203         ;LED 203 bit
        b       PortB_Write
LED3off:
        lac     PortBmirror
        andk    LED_203_OFF
        b       PortB_Write
LED4on: lac     PortBmirror
        ork     LED_204         ;LED 204 bit
        b       PortB_Write
LED4off:
        lac     PortBmirror
        andk    LED_204_OFF
        b       PortB_Write
FREQ_UP_1on:
        lac     PortBmirror
        ork     FREQ_UP_R1      ;freq up port 1
        b       PortB_Write
FREQ_UP_1off:
        lac     PortBmirror
        andk    ~FREQ_UP_R1
        b       PortB_Write
FREQ_UP_2on:
        lac     PortBmirror
        ork     FREQ_UP_R2      ;freq up port 2
        b       PortB_Write
FREQ_UP_2off:
        lac     PortBmirror
        andk    ~FREQ_UP_R2
        b       PortB_Write
FREQ_DN_1on:
        lac     PortBmirror
        ork     FREQ_DN_R1      ;freq down port 1
        b       PortB_Write
FREQ_DN_1off:
        lac     PortBmirror
        andk    ~FREQ_DN_R1
        b       PortB_Write
FREQ_DN_2on:
        lac     PortBmirror
        ork     FREQ_DN_R2      ;freq down port 2
        b       PortB_Write
FREQ_DN_2off:
        lac     PortBmirror
        andk    ~FREQ_DN_R2
PortB_Write:
        sacl    PortBmirror
        out     PortBmirror,RADIO_CTRL
        ret
;
;==============================================================================
;            ==========    D e l a y x ( AR0 )     ==========
;==============================================================================
;This routine is called with AR0 containing the delay interval in 5 mSec units.
;The routine waits until the Timer1 value decrements from the specified count
;to zero.  The Timer IRQ decrements the Timer1 value.
;
Delayx: larp	AR0
        lrlk    AR0,Timer1+USER_ADR
        sacl    *
Dlx_1:  zals    *
        bnz     Dlx_1
        ret
;
;=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
;  ======  I N T E R R U P T   S E R V I C E   R O U T I N E S  =========
;=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

;==============================================================================
;         ======    R x I n t S e r v i c e ( )      =======
;==============================================================================
;   Since the A/D and D/A are running at the same sample rate(10893 Hz),  only
;the receive interrupt is used for both.
;void RxIntService(){
;   Save_Context();
;   DXR = Sample_Que[AR6];              // write D/A from Sample_Que
;   Sample_Que[AR6++] = DRR;            // read A/D into Sample_Que
;   if( AR6>Sample_Que+SAMPQUESIZE-1 )  // deal with wrap around
;      AR6 = Sample_Que;
;   Restore_Context();
;}
RxIntService:                   ;SIO receive interrupt service routine
        larp    AR3             ;save context on software stack
        mar     *-              ;stackpointer--
        sst1    *-              ;push save status reg 1
        sst     *-              ;push save status reg 0
        sar     AR0,*,AR6       ;save AR0 and point to next sample in
;                                  Sample_Que
        ldpk    BASE_PAGE
        lar     AR0,*           ;AR0=next xmit sample
;$$$$$
;     ldpk    USER_PAGE          ;useful for outputing test data
;     lar     AR0,TestOut
;     ldpk    BASE_PAGE
;$$$$$
        sar     AR0,DXR         ;send to D/A converter for output
;
        lar     AR0,DRR         ;AR0=next A/D sample
        sar     AR0,*+          ;put in Sample_Que and inc AR6 head pointer
        lrlk    AR0,Sample_Que+SAMPQUESIZE-1    ;check for wrap around
        cmpr    CMGT            ;TC = (AR6>AR0)
        bbz     Rx1             ;br if no wrap
        lrlk    AR6,Sample_Que  ;else wrap it around
;
Rx1:    larp    AR3             ;restore context
        lar     AR0,*+          ;restore AR0
        lst     *+              ;pop status 0
        lst1    *+              ;pop status 1
        eint
        ret
;..............................................................................
;==============================================================================
;                 T x I n t S e r v i c e ( )
;==============================================================================
TxIntService:                   ;SIO transmit interrupt service routine
        lac     ToutBuf         ;used only during initialization
        sacl    DXR
        eint
        ret
;..............................................................................
;==============================================================================
;                 T i m e r I n t S e r v i c e ( )
;==============================================================================
;void TimerIntService(){
;   Save_Context();
;   if( Timer1 )                // decrement Timer1 down to zero
;      Timer1--;
;   if( Timer2 )                // decrement Timer2 down to zero
;      Timer2--;
;   Restore_Context();
;}
;
TimerIntService:
        larp    AR3             ;save context on software stack
        mar     *-              ;stackpointer--
        sst1    *-              ;push save status reg 1
        sst     *-              ;push save status reg 0
        sar     AR0,*           ;save AR0
;
        ldpk    USER_PAGE
        larp    AR0             ;ARP = AR0
        lar     AR0,Timer1      ;AR0 = Timer1
        banz    TI1,*-          ;br if( AR0-- != 0);
        b       TI2             ;else Timer1 is Zero so keep it zero
TI1:    sar     AR0,Timer1      ;Timer1--;
TI2:
        lar     AR0,Timer2      ;AR0 = Timer2
        banz    TI3,*-          ;br if( AR0-- != 0);
        b       TI4             ;else Timer2 is Zero so keep it zero
TI3:    sar     AR0,Timer2      ;Timer2--;
TI4:
        larp    AR3             ;restore context
        lar     AR0,*+          ;restore AR0
        lst     *+              ;pop status 0
        lst1    *+              ;pop status 1
        eint
        ret
;..............................................................................
;==============================================================================
;         =========    I n i t _ S y s t e m ( )    ==========
;==============================================================================
;This routine initializes the various hardware and software variables
;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . 
Init_System:
        ldpk    BASE_PAGE       ;data mem page 0 (internal)
        ssxm                    ;set sign extension mode
        sovm                    ;saturate on overflow
        spm     1               ;set P shift for Q15
        fort    0               ;AIO = 16 bit mode
        sfsm                    ;use external frame sync signal for receive
        rtxm                    ;use external frame sync signal for xmit
        cnfp                    ;B0 as program memory
;
        lrlk    AR3,Stack       ;init software stack        
;
        lalk    MS5PERIOD       ;Init 5 Millisecond Timer
        sacl    PRD             ;set up 5 millisecond timer
        lalk    TM_EN           ;make Timer interrupt active
        sacl    IMR             ;set INT mask
        eint                    ;enable interrupts
;
        ldpk    USER_PAGE       ;DEBUG stuff
        lalk    0h
        sacl    DEBUGPC         ;useage flag for debug routine( see TAPR docs.)
        lalk    0460h           ;point to first memory location used in page 8.
        sacl    DEBUGML
;
        zac
        sacl    Port6mirror
        sacl    Port7mirror
        sacl    PortBmirror
        sacl    Timer1
        sacl    Timer2

;................ Initialize A/D Converter chip........................
        call    AIO_Disable
        call    AIO_Reset
;
        call    LED2on          ;Blink all the LEDs
        call    LED3on
        call    LED4on
        call    LED5on
        call    LED6on
        call    LED7on
        lalk    50
        call    Delayx          ;delay 50 x .005 = 250 ms
        call    LED2off
        call    LED3off
        call    LED5off
        call    LED6off
        call    LED7off
;
        call    AIO_Enable
        call    AIO_Run
        lalk    20
        call    Delayx          ;delay 20 x .005 = 100 ms
;
;................ initialize AIO  .............................................
        ldpk    BASE_PAGE       ;data mem page 0 (internal)
;
        lalk    TX_EN           ;mask SIO TX interrupt active
        sacl    IMR             ;set INT mask for tx irq's
        zac
        sacl    ToutBuf         ;store data in DXR to transmit to AIO
        eint                    ;enable TX interrupt
        idle                    ;wait for TX interrupt
        lalk    AIO_CONFIG      ;load AIO command flag
        sacl    ToutBuf         ;store data in DXR to transmit to AIO
        idle                    ;wait for TX interrupt
        lalk    CMDC            ;load AIO config command
        sacl    ToutBuf         ;store data in DXR to transmit to AIO
        idle                    ;wait for TX interrupt
        lalk    AIO_CONFIG      ;load AIO command flag
        sacl    ToutBuf         ;store data in DXR to transmit to AIO
        idle                    ;wait for TX interrupt
        lalk    CMDB            ;load TB/RB AIO command next
        sacl    ToutBuf         ;store data in DXR to transmit to AIO
        idle                    ;wait for TX interrupt
        lalk    AIO_CONFIG      ;load AIO command flag
        sacl    ToutBuf         ;store data in DXR to transmit to AIO
        idle                    ;wait for TX interrupt
        lalk    CMDA            ;load TA/RA AIO command last
        sacl    ToutBuf         ;store data in DXR to transmit to AIO
        idle                    ;wait for TX interrupt
;
        dint                    ;disable interrupts
        lalk    RX_EN|TM_EN     ;make SIO RX and timer interrupt active
        sacl    IMR             ;store mask in IMR
        ldpk    USER_PAGE       ;data mem page 8 (external)
;
        lalk    RAND_SEED       ;initialize random number variables
        sacl    Random0
        lalk    RAND_K
        sacl    Rand_K
;
        zac
        sacl    Flags
        call    Init_Hin        ;initialize external modules
        call    Init_Hout
        call    Init_Kin
        call    Init_Kout
        call    Init_Ain
;
        eint                    ;enable interrupts
;
;. . . . . . . . . . U A R T   S E T U P . . . . . . . . . . . . . . . . .
;change rate to BAUDRATE Bits per second, 8 bits, no parity, one stop bit
        lalk    UART_SEL_LCR    ;select Line Control register
        sacl    Temp0
        out     Temp0,UART_CTRL
        in      Temp1,UART_READ ;read current Line Control values
        lac     Temp1
        ork     UART_LCR_DLAB
        sacl    Temp1
        out     Temp1,UART_WRITE
        lalk    UART_SEL_DL_MSB
        sacl    Temp0
        out     Temp0,UART_CTRL ;select MSB of buad rate divisor
        lalk    (BAUDRATE >> 8)
        sacl    Temp0
        out     Temp0,UART_WRITE
;
        lalk    UART_SEL_DL_LSB
        sacl    Temp0
        out     Temp0,UART_CTRL ;select LSB of buad rate divisor
        lalk    (BAUDRATE & 0FFh)
        sacl    Temp0
        out     Temp0,UART_WRITE
;
        lalk    UART_SEL_LCR    ;select Line Control register
        sacl    Temp0
        out     Temp0,UART_CTRL
;
        lac     Temp1           ;deselect baud rate divisor register
        andk    ~UART_LCR_DLAB
        sacl    Temp1
        out     Temp1,UART_WRITE
;
        lalk    UART_SEL_MCR    ;select Modem Control register
        sacl    Temp0
        out     Temp0,UART_CTRL
	lack    UART_MCR_RTS|UART_MCR_DTR
	sacl    Temp0
        out     Temp0,UART_WRITE
;
        ret
;..............................................................................
;    ============  E N D  o f   K S 1 2 M A I N . A S M  ===========
        .end
